Package: reems
Title: Estimating Effective Migration Surfaces from Single Nucleotide
        Polymorphism Data
Version: 0.1.0
Authors@R: c(
	   person(given = "Felix", family = "Weitkämper", email = "felix.weitkaemper@german-uds.de", role = c("aut", "cre","cph"), comment = c(ORCID = "0000-0002-3895-8279")),
	   person(given = "Desislava", family = "Petkova", role = c("cph"), comment = c(ORCID = "0000-0001-9129-1787", "Author of the 'EEMS' command-line utility and the 'EEMS' plotting 'R' scripts adapted for this wrapper package.")))
Description: Wrapper and plotting utilities for the spatial population genetics tool 'EEMS' (Estimated Effective Migration Surfaces) for SNP (Single Nucleotide Polymorphism) data, originally provided as a command-line tool written in 'C++' together with an accompanying 'R' package for plotting the output of the 'EEMS' tool itself (<https://github.com/dipetkov/eems/>). There are four main motivations for offering this to 'R' users as a package. Firstly, to remove the installation and configuration burden for the 'EEMS' command-line tool, which relies on manually installed 'Boost' and 'Eigen' system libraries and configuring their location; secondly, to streamline the workflow by having a singe environment (the 'R' system) for the entire analysis rather than a file-based command-line executable whose output files are then to be imported and analysed by a separate 'R' script; thirdly, to make the input formats compatible with other, 'R'-based spatial population genetics tools such as the 'ConStruct' package; and lastly, to allow for easily running several chains in parallel and combining them for plotting and further analysis. The package also adds more intuitive, streamlined tooling around creating more complex habitats. The method of estimating effective migration surfaces was first described by Petkova, D., Novembre, J. & Stephens, M. (2016) <doi:10.1038/ng.3464>.     
License: GPL-2
Encoding: UTF-8
RoxygenNote: 7.3.3
LinkingTo: BH (>= 1.76.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.9.1)
Imports: Rcpp, methods, raster (>= 2.4-15), sp (>= 1.1-1), sf, stats
        (>= 3.3.2), utils (>= 3.3.2)
Suggests: future, future.apply, Matrix (>= 1.2.7.1), RColorBrewer (>=
        1.1), deldir (>= 0.1), dichromat, rworldmap (>= 1.3),
        rworldxtra (>= 1.01), knitr, rmarkdown, testthat (>= 3.0.0)
Depends: R (>= 4.1.0)
LazyData: true
VignetteBuilder: knitr
URL: https://codeberg.org/felix_weitkaemper/reems
BugReports: https://codeberg.org/felix_weitkaemper/reems/issues
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-05-02 10:18:33 UTC; weitkaemper
Author: Felix Weitkämper [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-3895-8279>),
  Desislava Petkova [cph] (ORCID:
    <https://orcid.org/0000-0001-9129-1787>, Author of the 'EEMS'
    command-line utility and the 'EEMS' plotting 'R' scripts adapted
    for this wrapper package.)
Maintainer: Felix Weitkämper <felix.weitkaemper@german-uds.de>
Repository: CRAN
Date/Publication: 2026-05-05 18:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-05 21:38:47 UTC; unix
Archs: reems.so.dSYM

# Generated by roxygen2: do not edit by hand

export(eems)
export(eems.from.files)
export(eems.plots)
export(eems.population.grid)
export(eems.posterior.draws)
export(eems.voronoi.samples)
export(grid.make)
export(outer.buffered)
import(Rcpp)
import(grDevices)
import(graphics)
import(raster)
import(sf)
import(sp)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(utils,capture.output)
importFrom(utils,read.table)
importFrom(utils,write.table)
useDynLib(reems)

# reems

This R package implements the method of Estimating Effective Migration Surfaces (EEMS) described by Petkova, D., Novembre, J. & Stephens, M. Visualizing spatial population structure with estimated effective migration surfaces. Nat Genet 48, 94–100 (2016). https://doi.org/10.1038/ng.3464.

It provides the function `eems` to run an EEMS analysis, and the function `eems.plots` to visualize the estimated effective migration and diversity surfaces, as well as some other, more rarely used functions.

Its current stable version can be installed directly from CRAN, in the usual way. 
```R
install.packages("reems")
```
As this package provides a  wrapper around the original  C++ implementation of EEMS, installation of the package requires compilation of C++ source code.
This can lead to a lot of noisy console output, including several notes and warnings. This is expected and harmless, and as long as the installation process finishes successfully this is nothing to worry about. 

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## ----setup--------------------------------------------------------------------
# library(reems)

## -----------------------------------------------------------------------------
# eems(
#   freqs = freqs,
#   coords = coords,
#   mcmcpath = "/full/path/to/directory/prefix"
# )	

## -----------------------------------------------------------------------------
# eems(
#   seed = 1024,	
#   freqs = freqs,
#   coords = coords,
#   mcmcpath = "/full/path/to/directory/prefix",
#   diploid = FALSE,
#   numMCMCIter = 100000,
#   numBurnIter = 50000,
#   numThinIter = 999
# )	

## -----------------------------------------------------------------------------
# eems(
#   freqs = freqs,
#   coords = coords,
#   mcmcpath = "full/path/to/directory/prefix",
#   nChains = 5,
#   parallel = TRUE
# )	

## -----------------------------------------------------------------------------
# data_path <- system.file("extdata", package = "reems")
# input <- file.path(data_path, "barrier-schemeX-nIndiv300-nSites3000")
# 
# outer <- read.table(paste0(input, ".outer"))
# 
# hole1 <- data.frame(V1 = c(2., 5., 5., 2., 2.), V2 = c(2., 2., 5., 5., 2.))
# hole2 <- data.frame(V1 = c(6.5, 10., 8., 6.5), V2 = c(2.5, 5., 5., 2.5))
# 
# # Create the new grid  with holes
# grid <- grid.make(outer = outer, holes = list(hole1, hole2), nDemes = 300)
# 
# eems(
#   freqs = freqs,
#   coords = coords,
#   mcmcpath = "full/path/to/directory/prefix",
#   demes = grid$demes,
#   edges = grid$edges,
#   nChains = 5,
#   parallel = TRUE
# )	
# 

## -----------------------------------------------------------------------------
# ?eems

## -----------------------------------------------------------------------------
# eems_results <- eems(
#   freqs = freqs,
#   coords = coords,
#   mcmcpath = "/full/path/to/directory/prefix",
#   nChains = 5,
#   parallel = TRUE
# )	
# 
# eems.plots(
#   mcmcpath = eems_results,
#   plotpath = "/full/path/to/file/prefix",
#   longlat = TRUE
# )

## -----------------------------------------------------------------------------
# eems.plots(
#   mcmcpath = eems_results,
#   plotpath = "/full/path/to/file/with-sampling",
#   longlat = TRUE,
#   m.plot.xy = {
#     points(coord, col = "purple")
#   },
#   q.plot.xy = {
#     points(coord, col = "purple")
#   }
# )

## -----------------------------------------------------------------------------
# eems.plots(
#   mcmcpath = eems_results,
#   plotpath =  "/full/path/to/file/axes-flipped",
#   longlat = FALSE
# )

## -----------------------------------------------------------------------------
# eems.plots(
#   mcmcpath = eems_results,
#   plotpath =  "/full/path/to/file/output-PDFs",
#   longlat = TRUE,
#   plot.height = 12,
#   plot.width = 10
# )

## -----------------------------------------------------------------------------
# eems.plots(
#   mcmcpath = eems_results,
#   plotpath =  "/full/path/to/file/output-PNGs",
#   longlat = TRUE,
#   plot.height = 9,
#   plot.width = 8,
#   res = 600,
#   out.png = TRUE
# )

## -----------------------------------------------------------------------------
# eems.plots(
#   mcmcpath = eems_results,
#   plotpath =  "/full/path/to/file/demes-and-edges",
#   longlat = TRUE,
#   add.grid = TRUE,
#   col.grid = "gray90",
#   lwd.grid = 2,
#   add.outline = TRUE,
#   col.outline = "blue",
#   lwd.outline = 5,
#   add.demes = TRUE,
#   col.demes = "red",
#   pch.demes = 5,
#   min.cex.demes = 0.5,
#   max.cex.demes = 1.5
# )

## -----------------------------------------------------------------------------
# eems.plots(
#   mcmcpath = eems_results,
#   plotpath = "/full/path/to/file/new-eems-colors",
#   longlat = TRUE,
#   eems.colors = RColorBrewer::Rbrewer.pal(11, "RdBu")
# )

## -----------------------------------------------------------------------------
# eems.plots(
#   mcmcpath = eems_results,
#   plotpath = "/full/path/to/file/fix-colscales",
#   longlat = TRUE,
#   add.outline = TRUE,
#   col.outline = "gray",
#   m.colscale = c(-3, 3),
#   q.colscale = c(-0.3, +0.3)
# )

## -----------------------------------------------------------------------------
# projection_none <- "+proj=longlat +datum=WGS84"
# projection_mercator <- "+proj=merc +datum=WGS84"
# 
# eems.plots(
#   mcmcpath = eems_results,
#   plotpath = "/full/path/to/file/cartographic-projections",
#   longlat = TRUE,
#   projection.in = projection_none,
#   projection.out = projection_mercator
# )

## -----------------------------------------------------------------------------
# eems.plots(
#   mcmcpath = eems_results,
#   plotpath = "/full/path/to/file/geographic-map",
#   longlat = TRUE,
#   projection.in = projection_none,
#   projection.out = projection_mercator,
#   add.map = TRUE,
#   col.map = "black",
#   lwd.map = 5
# )

## -----------------------------------------------------------------------------
# map_world <- rworldmap::getMap()
# map_africa <- map_world[which(map_world@data$continent == "Africa"), ]
# eems.plots(
#   mcmcpath = eems_results,
#   plotpath = "/full/path/to/file/shapefile",
#   longlat = TRUE,
#   m.plot.xy = {
#     plot(map_africa, col = NA, add = TRUE)
#   },
#   q.plot.xy = {
#     plot(map_africa, col = NA, add = TRUE)
#   }
# )

## -----------------------------------------------------------------------------
# map_africa <- sp::spTransform(map_africa,  sp::CRS(projection_mercator))
# eems.plots(
#   mcmcpath = eems_results,
#   plotpath =  "/full/path/to/file/shapefile-projected",
#   longlat = TRUE,
#   projection.in = projection_none,
#   projection.out = projection_mercator,
#   m.plot.xy = {
#     plot(map_africa, col = NA, add = TRUE)
#   },
#   q.plot.xy = {
#     plot(map_africa, col = NA, add = TRUE)
#   }
# )

## -----------------------------------------------------------------------------
# coords <- matrix(c(
#   -10, 10,
#   10, 10,
#   30, 0,
#   40, -10,
#   30, -20
# ), ncol = 2, byrow = TRUE)
# colors <- c("red", "green", "blue", "purple", "orange")
# labels <- LETTERS[1:5]
# coords_merc <- sp::spTransform(
#   sp::SpatialPoints(coords, sp::CRS(projection_none)),
#   sp::CRS(projection_mercator)
# )
# 
# coords_merc <- coords_merc@coords
# 
# eems.plots(
#   mcmcpath = eems_results,
#   plotpath = "/full/path/to/file/labels-projected",
#   longlat = TRUE,
#   projection.in = projection_none,
#   projection.out = projection_mercator,
#   m.plot.xy = {
#     text(coords_merc, col = colors, pch = labels, font = 2)
#   },
#   q.plot.xy = {
#     text(coords_merc, col = colors, pch = labels, font = 2)
#   }
# )

## -----------------------------------------------------------------------------
# xy.coords <- matrix(c(
#   13.70, 3.20,
#   37.10, -7.20,
#   36.10, -4.10,
#   34.58, -5.67
# ), ncol = 2, byrow = TRUE)
# eems.plots(
#   mcmcpath = eems_results,
#   plotpath = "/full/path/to/file/default",
#   longlat = TRUE,
#   xy.coords = xy.coords
# )
# load(paste0("/full/path/to/file/default", "-rdist.RData"))

## -----------------------------------------------------------------------------
# ?eems.plots

#' A function to plot the constructed population grid and optionally the initial sampling locations
#'
#' Given an EEMS output directory, this function generates one figure to visualize the EEMS
#' population grid. All edges are shown in the same color to visualize the grid before estimating
#' migration and diversity rates. This can be helpful if EEMS exits with the error message
#' "The population grid is not connected".
#' @param add.coord A logical value indicating whether to add the original sampling locations to
#' the plot or not.
#' @param col.coord The color of the sampling locations. Defaults to \code{red}.
#' @param pch.coord The symbol, specified as an integer, or the character to be used for plotting
#' the sampling locations. Defaults to 3.
#' @param datapath The full path and the file name of the input dataset (the three files
#' datapath.coord, datapath.diffs, datapath.outer). Must be specified if \code{add_coord = TRUE}.
#' @param out.png A logical value which, if set,  forces output graphics to be generated as PNGs
#' (if `TRUE`) or PDFs (if `FALSE`). Defaults to `FALSE`.
#' @inheritParams eems.plots
#' @returns Passes on the return value of [dev.off()] indicating the graphics device.  
#' @examples
#' # Use the provided example or supply the path to your own EEMS run.
#' extdata_path <- system.file("extdata", package = "reems")
#' eems_results <- file.path(extdata_path, "EEMS-example")
#' # Create a temporary output directory for this example 
#' outdir <- file.path(tempdir(), "path_out")
#' dir.create(outdir, showWarnings = FALSE)
#' name_figures <- file.path(outdir, "EEMS-grid_connected")
#'
#' eems.population.grid(eems_results,
#'   name_figures,
#'   longlat = TRUE,
#'   add.outline = TRUE,
#'   col.outline = "purple",
#'   lwd.outline = 3,
#'   add.grid = TRUE,
#'   col.grid = "green",
#'   lwd.grid = 2,
#'   out.png = FALSE
#' )
#'
#' # It is more interesting to see an example where the grid is unconnected
#' # due to the unusual shape of the habitat.
#' eems_results <- file.path(extdata_path, "EEMS-popgrid")
#' 
#' name_figures <- file.path(outdir, "EEMS-grid_not_connected")
#'
#' eems.population.grid(
#'   mcmcpath = eems_results,
#'   plotpath = name_figures,
#'   longlat = FALSE,
#'   add.outline = TRUE, col.outline = "purple", lwd.outline = 3,
#'   add.grid = TRUE, col.grid = "green", lwd.grid = 2
#' )
#' # Delete the output file to tidy up.
#' unlink(outdir, recursive = TRUE, force = TRUE)
#' @seealso \code{\link{eems.plots}, \link{eems.voronoi.samples}, \link{eems.posterior.draws}}
#' @export

eems.population.grid <- function(mcmcpath,
                                 plotpath,
                                 longlat,
                                 plot.width = 10,
                                 plot.height = 10,
                                 out.png = FALSE,
                                 res = 600,
                                 add.grid = TRUE,
                                 col.grid = "gray80",
                                 lwd.grid = 1,
                                 add.demes = FALSE,
                                 col.demes = "black",
                                 pch.demes = 19,
                                 min.cex.demes = 1,
                                 max.cex.demes = 3,
                                 add.outline = TRUE,
                                 col.outline = "gray90",
                                 lwd.outline = 2,
                                 add.coord = FALSE,
                                 col.coord = "red",
                                 pch.coord = 3,
                                 datapath = NULL) {
  plot.params <- list(
    add.grid = add.grid, add.outline = add.outline, add.demes = add.demes,
    col.grid = col.grid, col.outline = col.outline, col.demes = col.demes,
    lwd.grid = lwd.grid, lwd.outline = lwd.outline, pch.demes = pch.demes,
    min.cex.demes = min.cex.demes, max.cex.demes = max.cex.demes
  )
  plot.params <- check.plot.params(plot.params)

  # A vector of EEMS output directories, for the same dataset.
  # Assume that if eemsrun.txt exists, then all EEMS output files exist.
  mcmcpath <- mcmcpath[file.exists(file.path(mcmcpath, "eemsrun.txt"))]
  if (!length(mcmcpath)) {
    stop("Please provide at least one existing EEMS output directory, mcmcpath")
  }

  save.params <- list(height = plot.height, width = plot.width, res = res, out.png = out.png)

  message("Processing the following EEMS output directories :")
  message(mcmcpath)

  # Ensure graceful exit closing graphics devices.
  initial_ndev <- length(grDevices::dev.list())
  on.exit({
      while (!is.null(grDevices::dev.list()) &&
             length(grDevices::dev.list()) > initial_ndev) {
                 dev.off()
             }
  }, add = TRUE)

  save.graphics(paste0(plotpath, "-popgrid"), save.params)

  # Read the habitat outline from the mcmcpath (output) directory
  graph <- read.graph(mcmcpath, longlat)
  outer <- graph$outer

  # Read the sampling locations from the datapath (input) directory
  coordpath <- paste0(datapath, ".coord")
  coord <- NULL

  if (!is.null(datapath) && file.exists(coordpath)) {
    message(paste0("Read the original sampling locations from ", coordpath))
    coord <- scan(coordpath, what = numeric(), quiet = TRUE)
    coord <- matrix(coord, ncol = 2, byrow = TRUE)
    if (!longlat) {
      coord <- coord[, c(2, 1)]
    }
  }

  # Make the plot canvas large enough to fit both the sampling locations and the habitat outline
  plot(rbind(outer, coord), xlab = "", ylab = "", axes = FALSE, type = "n", asp = 1)

  filled.contour.outline(mcmcpath, longlat, plot.params)
  filled.contour.graph(mcmcpath, longlat, plot.params)

  # Plot the sampling locations
  if (add.coord) {
    if (is.null(coord)) {
      message(
        "\nTo add the sampling locations with option add.coord = TRUE,\n",
        "Specify the full path to the input dataset (coord/diffs/outer files)\n\n"
      )
    } else {
      points(coord, pch = pch.coord, col = col.coord)
    }
  }
  dev.off()
}

test_that("eems.population.grid writes output", {
    extdata_path <- system.file("extdata", package = "reems")
    eems_results <- file.path(extdata_path, "EEMS-example")
    outdir <- file.path(tempdir(), "path_out")
    dir.create(outdir, showWarnings = FALSE)
    name_figures <- file.path(outdir, "EEMS-grid_connected")

    eems.population.grid(eems_results,
                         name_figures,
                         longlat = TRUE,
                         add.outline = TRUE, col.outline = "purple", lwd.outline = 3,
                         add.grid = TRUE, col.grid = "green", lwd.grid = 2
                         )
    
    
    files <- list.files(outdir, full.names = TRUE)
    expect_length(files, 1)
    expect_true(all(file.info(files)$size > 0))
    unlink(outdir, recursive = TRUE, force = TRUE)
})

    

#' Create a habitat with holes
#' 
#'  `grid.make()` constructs a regular triangular grid with circa nDemes vertices. This function creates three files:
#'  * The pair gridpath.demes and gridpath.edges which specify a custom population grid with holes
#'  * The plot gridpath.png which visualizes the custom grid with demes in red and edges in green
#' @param outer A two-column matrix of coordinates of boundary points (longitude, latitude).
#' @param holes List of holes. Each hole is a two-column matrix of coordinates of boundary points (longitude, latitude).
#' This is optional, and defaults to no holes.
#' @param nDemes Number of demes, roughly, in the grid without accounting for possible holes.
#' @param plotpath Output filename for the visualisation PNG file. Optional.
#' @returns A two-entry list of `demes` and `edges`, both two-column matrices.
#' @examples
#' # We use example input from Petkova et al. (2016), found in the '/extdata' directory
#' data_path <- system.file("extdata", package = "reems")
#' input <- file.path(data_path, "barrier-schemeX-nIndiv300-nSites3000")
#' # Load the population habitat
#' outer <- read.table(paste0(input, ".outer"))
#' # Each hole is a ring (simple closed polygon) and the holes don't overlap
#' hole1 <- data.frame(V1 = c(2., 5., 5., 2., 2.), V2 = c(2., 2., 5., 5., 2.))
#' hole2 <- data.frame(V1 = c(6.5, 10., 8., 6.5), V2 = c(2.5, 5., 5., 2.5))
#'
#' # Create the new grid  with holes
#' grid.make(outer = outer, holes = list(hole1, hole2), nDemes = 300)
#' @export
grid.make <- function(outer, holes = NULL, plotpath = NULL, nDemes) {
    data_path <- file.path(tempdir(), "data_path")
    dir.create(data_path, showWarnings = FALSE)
    on.exit(unlink(data_path, recursive = TRUE, force = TRUE), add = TRUE)
    datapath <- file.path(data_path,"in")
    outerfile <- paste0(datapath,".outer")
    write_for_eems(outer,outerfile)
    mcmcpath <-  file.path(tempdir(), "intermediate_path")
    dir.create(mcmcpath, showWarnings = FALSE)
    on.exit(unlink(mcmcpath, recursive = TRUE, force = TRUE), add = TRUE)
    cpp_write_grid(datapath, mcmcpath, nDemes)
    if (is.null(holes)){
        demes <- read.table(file.path(mcmcpath, "demes.txt"))
        edges <- read.table(file.path(mcmcpath, "edges.txt"))
        if (!is.null(plotpath)) {
            plot_basic_popgrid(demes, edges, plotpath)
        }
        return(list("demes" = demes, "edges" = edges))
    }
    else {
        habitat_with_holes <- create_habitat_with_holes(outer, holes)
        outlist <- remove_demes_outside_habitat(habitat_with_holes, mcmcpath, plotpath)
        return(outlist)
    }
}
        
    
#' Create a habitat with holes
#' @param outer A two-column matrix of coordinates of boundary points (longitude, latitude).
#' @param holes List of holes. Each hole is a two-column matrix of coordinates of boundary points.
#' (longitude, latitude).
#' @returns A spatial polygon object with the outer boundary `outer` and the holes in `holes`.  
#' @examples
#' # The input is an EEMS run with the default regular triangular grid without holes
#' extdata <- system.file("extdata", package = "reems")
#' eems_run_with_default_habitat <- file.path(extdata, "EEMS-barrier")
#' # Load the population habitat
#' outer <- read.table(file.path(eems_run_with_default_habitat, "outer.txt"))
#' # Each hole is a ring (simple closed polygon) and the holes don't overlap
#' hole1 <- data.frame(V1 = c(2., 5., 5., 2., 2.), V2 = c(2., 2., 5., 5., 2.))
#' hole2 <- data.frame(V1 = c(6.5, 10., 8., 6.5), V2 = c(2.5, 5., 5., 2.5))
#'
#' # Create the new habitat with holes
#' create_habitat_with_holes(outer, list(hole1, hole2))
#' @noRd
create_habitat_with_holes <- function(outer, holes) {
  polys <- list(Polygon(outer, FALSE))

  for (hole in holes) {
    polys <- c(polys, Polygon(hole, TRUE))
  }

  SpatialPolygons(list(Polygons(polys, "1")))
}

point_is_in_habitat <- function(point, habitat) {
  point <- matrix(point, nrow = 1)
  point <- SpatialPoints(point)
  point <- point[habitat]
  length(point) == 1
}

reindex_demes_sequentially <- function(in_habitat) {
  indices_in_habitat <- cumsum(in_habitat)
  indices_in_habitat[!in_habitat] <- NA_integer_
  indices_in_habitat
}

reindex_demes <- function(demes, new_indices) {
  demes[!is.na(new_indices), ]
}

reindex_edges <- function(edges, new_indices) {
  edges[, 1] <- new_indices[edges[, 1]]
  edges[, 2] <- new_indices[edges[, 2]]
  na.omit(edges)
}

plot_basic_popgrid <- function(demes, edges, gridpath, col.grid = "green", col.demes = "red", ...) {
  # Ensure graceful exit closing graphics devices.
    initial_ndev <- length(grDevices::dev.list())
    on.exit({
        while (!is.null(grDevices::dev.list()) &&
               length(grDevices::dev.list()) > initial_ndev) {
                   dev.off()
               }
    }, add = TRUE)
    
    bitmap(
        paste0(gridpath, ".png"),
        type = "png16m", res = 600, units = "in", height = 6, width = 8, ...
    )
    
    plot(demes, col = col.demes, pch = 19, xlab = "", ylab = "")
    
    for (i in seq_len(nrow(edges))) {
        a <- edges[i, 1]
        b <- edges[i, 2]
        lines(demes[c(a, b), ], col = col.grid)
    }
    dev.off()
}

#' Remove demes that lie outside of the habitat
#'
#' If `plotpath` is provided, this function creates a file `plotpath.png`,
#' which visualizes the custom grid with demes in red and edges in green.
#' 
#' @param habitat A SpatialPolygons habitat (possibly with holes).
#' @param mcmcpath An EEMS output directory which contains demes.txt and edges.txt.
#' @param plotpath Output filename for the visualisation PNG file. Optional.
#' @returns A two-entry list of `demes` and `edges`, both two-column matrices.
#' @examples
#' # The input is an EEMS run with the default regular triangular grid without holes
#' extdata <- system.file("extdata", package = "reems")
#' eems_run_with_default_habitat <- file.path(extdata, "EEMS-barrier")
#' # The output filepath (in the temporary directory for our example);
#' # three files with the same name but different extension to be created
#' outdir <- file.path(tempdir(), "path_out")
#' dir.create(outdir, showWarnings = FALSE)
#' custom_grid_path_with_holes <- file.path(outdir, "gridpath-with-holes")
#'
#' # Load the population habitat
#' outer <- read.table(file.path(eems_run_with_default_habitat, "outer.txt"))
#' # Each hole is a ring (simple closed polygon) and the holes don't overlap
#' hole1 <- data.frame(V1 = c(2., 5., 5., 2., 2.), V2 = c(2., 2., 5., 5., 2.))
#' hole2 <- data.frame(V1 = c(6.5, 10., 8., 6.5), V2 = c(2.5, 5., 5., 2.5))
#'
#' # Create the new habitat with holes
#' new_habitat_with_holes <- create_habitat_with_holes(outer, list(hole1, hole2))
#'
#' remove_demes_outside_habitat(
#'   habitat = new_habitat_with_holes,
#'   mcmcpath = eems_run_with_default_habitat,
#' )
#' # Delete the output directory to tidy up. 
#' unlink(outdir, recursive = TRUE, force = TRUE)
#' @noRd
remove_demes_outside_habitat <- function(habitat, mcmcpath, plotpath = NULL) {
  demes <- read.table(file.path(mcmcpath, "demes.txt"))
  edges <- read.table(file.path(mcmcpath, "edges.txt"))

  in_habitat <- apply(demes, 1, point_is_in_habitat, habitat)
  new_indices <- reindex_demes_sequentially(in_habitat)

  demes <- reindex_demes(demes, new_indices)
  edges <- reindex_edges(edges, new_indices)

  demes <- as.matrix(demes)
  edges <- as.matrix(edges)

  if (!is.null(plotpath)) {
      plot_basic_popgrid(demes, edges, plotpath)
  }
  return(list("demes" = demes, "edges" = edges)) 
}

test_that("grid.make returns a list of demes and edges", {
    data_path <- system.file("extdata", package = "reems")
    input <- file.path(data_path, "barrier-schemeX-nIndiv300-nSites3000")
    outer <- read.table(paste0(input, ".outer"))
    hole1 <- data.frame(V1 = c(2., 5., 5., 2., 2.), V2 = c(2., 2., 5., 5., 2.))
    hole2 <- data.frame(V1 = c(6.5, 10., 8., 6.5), V2 = c(2.5, 5., 5., 2.5))
    
    output <- grid.make(outer = outer, holes = list(hole1, hole2), nDemes = 300)
    expect_true(is.matrix(output$demes))
    expect_true(is.matrix(output$edges))
    expect_equal(ncol(output$demes), 2)
    expect_equal(ncol(output$edges), 2)
})

 

plot.logposterior <- function(mcmcpath) {
  message("Plotting posterior probability trace")
  nchains <- length(mcmcpath)
  # Here colors = RColorBrewer::brewer.pal(12, "Paired") + "black"
  colors <- rep_len(c(
    "#000000", "#A6CEE3", "#1F78B4", "#B2DF8A", "#33A02C", "#FB9A99", "#E31A1C", "#FDBF6F",
    "#FF7F00", "#CAB2D6", "#6A3D9A", "#FFFF99", "#B15928"
  ), length.out = nchains)
  ltypes <- rep_len(1:4, length.out = nchains)
  posteriors <- list()
  yrange <- NULL
  niters <- NULL
  for (i in 1:nchains) {
    path <- mcmcpath[i]
    message(path)
    stopifnot(all(file.exists(file.path(path, "mcmcpilogl.txt"))))
    pilogl <- scan(file.path(path, "mcmcpilogl.txt"), quiet = TRUE)
    pilogl <- matrix(pilogl, ncol = 2, byrow = TRUE)
    posterior <- pilogl[, 1] + pilogl[, 2]
    posteriors[[i]] <- posterior
    yrange <- range(c(yrange, posterior))
    niters <- max(niters, length(posterior))
  }
  plot(c(1, niters), yrange,
    type = "n",
    xlab = "MCMC iteration  (after burn-in and thinning)",
    ylab = "log posterior"
  )
  if (nchains == 1) {
    mtext(
      side = 3, line = 2, cex = 1.3,
      text = "Has the MCMC chain converged?"
    )
    mtext(
      side = 3, line = 0.5, cex = 1,
      text = "If not, restart EEMS and/or increase numMCMCIter, numBurnIter, numThinIter"
    )
  } else {
    mtext(
      side = 3, line = 2, cex = 1.3,
      text = "Have the MCMC chains converged?"
    )
    mtext(
      side = 3, line = 0.5, cex = 1,
      text = "If not, restart EEMS and/or increase numMCMCIter, numBurnIter, numThinIter"
    )
  }
  for (i in 1:nchains) {
    posterior <- posteriors[[i]]
    niters <- length(posterior)
    lines(1:niters, posterior, col = colors[i], lty = ltypes[i], lwd = 2)
  }
  legend("topright",
    legend = 1:nchains, col = colors[1:nchains],
    lty = ltypes[1:nchains], lwd = 2, bty = "n", inset = c(-0.12, 0), cex = 0.5
  )
}

.onAttach <- function(libname, pkgname) {
  packageStartupMessage(
    "reems is an R wrapper for the EEMS spatial population genetics method.\n",
    "It also integrates the R plotting functions bundled with the original EEMS command-line utility.\n",
    "reems uses DarkOrange to Blue color scheme, with 'white' as the midpoint color.\n",
    "It combines two color schemes from the 'dichromat' package, which itself is based on\n",
    "a collection of color schemes for scientific data graphics:\n",
    "\tLight A and Bartlein PJ (2004).\n",
    "\tThe End of the Rainbow? Color Schemes for Improved Data Graphics.\n",
    "\tEOS Transactions of the American Geophysical Union, 85(40), 385.\n",
    "See also http://geog.uoregon.edu/datagraphics/color_scales.htm\n\n\n"
  )
}

default.eems.colors <- function() {
  c(
    # orange sequence, from dark to light
    "#994000", "#CC5800", "#FF8F33", "#FFAD66", "#FFCA99", "#FFE6CC",
    # very slightly off-white
    "#FBFBFB",
    # blue sequence, from light to dark
    "#CCFDFF", "#99F8FF", "#66F0FF", "#33E4FF", "#00AACC", "#007A99"
  )
}

is.color <- function(x) {
  if (is.null(x)) {
    return(FALSE)
  }
  # Check that a string, or a vector of strings, represents a valid color
  sapply(x, function(x) {
    tryCatch(is.matrix(col2rgb(x)), error = function(e) FALSE)
  })
}

set.colscale <- function(x) {
  if (min(x) < max(x)) range(x) else c(-1, 1)
}

mypalette <- function(colors = NULL, colscale = NULL, n = 299) {
  if (is.null(colors)) {
    # Use RColorBrewer's 9-color "Reds" palette.
    colors <- c(
      "#FFF5F0", "#FEE0D2", "#FCBBA1", "#FC9272", "#FB6A4A",
      "#EF3B2C", "#CB181D", "#A50F15", "#67000D"
    )
  }
  colors <- colorRampPalette(colors)(n = n)

  if (!is.null(colscale)) {
    if (length(colscale) != (n + 1)) {
      levels <- seq(from = min(colscale), to = max(colscale), length.out = n + 1)
    } else {
      levels <- sort(colscale)
    }
  }
  list(colors = colors, levels = levels)
}

eems.colscale <- function(values, nlevels, colscale) {
  max_val <- max(values + 0.001, colscale)
  min_val <- min(values - 0.001, colscale)
  seq(from = min_val, to = max_val, length = nlevels + 1)
}

#' Plotting smoothed Voronoi diagrams of effective migration and diversity rates
#'
#' Given a set of EEMS output directories, this function takes random draws from the posterior
#' distribution of the migration and diversity rate parameters. Each draw is visualized as two
#' smoothed Voronoi diagrams; the migration diagram is saved to a file ending in \code{mvoronoiXX},
#' the diversity diagram is saved to a file ending in \code{qvoronoiXX} where \code{XX} is a
#' numeric id. Specify the number of times to draw from the posterior with the argument
#' \code{post.draws}. If \code{post.draws = 10}, then \code{eems.posterior.draws} will generate
#' plots with id \code{XX = 1} to \code{XX = 10}.
#' This function differs from [eems.voronoi.samples()] by plotting smoothed contour diagrams
#' rather than raw Voronoi plots. 
#'
#' Note about the implementation: \code{eems.voronoi.samples} samples randomly from the posterior
#' draws saved during the execution of EEMS, after burn-in and thinning.
#' @param post.draws Number of times to sample from the posterior. The default is 1.
#' @param  out.png A logical value which, if set,  forces output graphics to be generated as PNGs
#' (if `TRUE`) or PDFs (if `FALSE`). Defaults to `TRUE`.
#' @inheritParams eems.plots
#' @returns None  
#' @references Light A and Bartlein PJ (2004). The End of the Rainbow? Color Schemes for Improved
#' Data Graphics. EOS Transactions of the American Geophysical Union, 85(40), 385.
#' @examples
#' # Use the provided example or supply the path to your own EEMS run.
#' extdata_path <- system.file("extdata", package = "reems")
#' eems_results <- file.path(extdata_path, "EEMS-example")
#' # Create a temporary output directory for this example 
#' outdir <- file.path(tempdir(), "path_out")
#' dir.create(outdir, showWarnings = FALSE)
#' name_figures <- file.path(outdir, "EEMS-example")
#'
#' # Plot a series of Voronoi diagrams for the EEMS model parameters:
#' # the effective migration rates (m) and the effective diversity rates (q).
#' eems.posterior.draws(
#'   mcmcpath = eems_results,
#'   plotpath = paste0(name_figures, "-posterior-draws"),
#'   longlat = TRUE,
#'   post.draws = 2,
#'   out.png = FALSE
#' )
#' # Delete the output directory to tidy up. 
#' unlink(outdir, recursive = TRUE, force = TRUE)
#' @seealso \code{\link{eems.voronoi.samples}}
#' @export

eems.posterior.draws <- function(mcmcpath,
                                 plotpath,
                                 longlat,
                                 post.draws = 1,
                                 plot.width = 10,
                                 plot.height = 10,
                                 out.png = TRUE,
                                 res = 600,
                                 xpd = TRUE,
                                 add.grid = FALSE,
                                 col.grid = "gray80",
                                 lwd.grid = 1,
                                 add.demes = FALSE,
                                 col.demes = "black",
                                 pch.demes = 19,
                                 min.cex.demes = 1,
                                 max.cex.demes = 3,
                                 add.outline = FALSE,
                                 col.outline = "gray90",
                                 lwd.outline = 2,
                                 projection.in = NULL,
                                 projection.out = NULL,
                                 add.map = FALSE,
                                 col.map = "gray60",
                                 lwd.map = 2,
                                 eems.colors = NULL,
                                 add.colbar = FALSE,
                                 m.colscale = NULL,
                                 q.colscale = NULL,
                                 add.title = FALSE,
                                 m.plot.xy = NULL,
                                 q.plot.xy = NULL) {
  plot.params <- list(
    eems.colors = eems.colors, m.colscale = m.colscale, q.colscale = q.colscale,
    add.map = add.map, add.grid = add.grid, add.outline = add.outline, add.demes = add.demes,
    col.map = col.map, col.grid = col.grid, col.outline = col.outline, col.demes = col.demes,
    lwd.map = lwd.map, lwd.grid = lwd.grid, lwd.outline = lwd.outline, pch.demes = pch.demes,
    min.cex.demes = min.cex.demes, proj.in = projection.in, add.colbar = add.colbar,
    max.cex.demes = max.cex.demes, proj.out = projection.out, add.title = add.title
  )
  plot.params <- check.plot.params(plot.params)

  # A vector of EEMS output directories, for the same dataset.
  # Assume that if eemsrun.txt exists, then all EEMS output files exist.
  mcmcpath <- mcmcpath[file.exists(file.path(mcmcpath, "eemsrun.txt"))]
  if (!length(mcmcpath)) {
    stop("Please provide at least one existing EEMS output directory, mcmcpath")
  }

  dimns <- read.dimns(mcmcpath[1], longlat)

  save.params <- list(height = plot.height, width = plot.width, res = res, out.png = out.png)

  plot_with_graphics_device(
      paste0(plotpath, "-mvoronoi"),
      save.params,
      for (draw in seq(post.draws)) {
                                        # Choose one output directory at random
          mrates.raster <- random.eems.contour(sample(mcmcpath, 1), dimns, longlat,
                                               plot.params,
                                               is.mrates = TRUE, plot.xy = m.plot.xy
                                               )
      },
      par.args = list(las = 1, font.main = 1)
  )
                                        # Graphics updated to here
   plot_with_graphics_device(
       paste0(plotpath, "-qvoronoi"),
       save.params,
       for (draw in seq(post.draws)) {
                                        # Choose one output directory at random
           qrates.raster <- random.eems.contour(sample(mcmcpath, 1), dimns, longlat,
                                                plot.params,
                                                is.mrates = FALSE, plot.xy = m.plot.xy
                                                )
       },
       par.args = list(las = 1, font.main = 1)
   )
invisible(NULL)
}

#' A function to plot effective migration and diversity surfaces from EEMS output
#'
#' Given a vector of EEMS output directories, this function generates several figures
#' to visualize EEMS results. It is a good idea to examine all these figures, which is
#' why they are generated by default.
#' \itemize{
#'   \item \code{plotpath-mrates01}: effective migration surface. This contour plot visualizes
#'   the estimated effective migration rates \code{m}, on the log10 scale after mean centering.
#'   \item \code{plotpath-mrates02}: posterior probabilities \code{P(m > 0 | diffs)} and
#'   \code{P(m < 0 | diffs)} for each location in the habitat. Since migration rates are
#'   visualized on the log10 scale after mean centering, 0 corresponds to the overall mean
#'   migration rate. This contour plot emphasizes regions with effective migration that is
#'   significantly higher/lower than the overall average.
#'   \item \code{plotpath-qrates01}: effective diversity surface. This contour plot visualizes
#'   the estimated effective diversity rates \code{q}, on the log10 scale after mean centering.
#'   \item \code{plotpath-qrates02}: posterior probabilities \code{P(q > 0 | diffs)} and
#'   \code{P(q < 0 | diffs)}. Similar to \code{plotpath-mrates02} but applied to the effective
#'   diversity rates.
#'   \item \code{plotpath-rdist01}: scatter plot of the observed vs the fitted between-deme
#'   component of genetic dissimilarity, where one point represents a pair of sampled demes.
#'   \item \code{plotpath-rdist02}: scatter plot of the observed vs the fitted within-deme
#'   component of genetic dissimilarity, where one point represents a sampled deme.
#'   \item \code{plotpath-rdist03}: scatter plot of observed genetic dissimilarities between
#'   demes vs observed geographic distances between demes.
#'   \item \code{plotpath-pilogl01}: posterior probability trace
#' }
#' The \code{mrates} and \code{qrates} figures visualize (properties of) the effective migration
#' and diversity rates across the habitat. The other figures can help to check that the MCMC
#' sampler has converged (the trace plot \code{pilogl}) and that the EEMS model fits the data
#' well (the scatter plots of genetic dissimilarities \code{rdist}).
#'
#' The function \code{eems.plots} will work given the results from a single EEMS run (one
#' directory in \code{mcmcpath}) but it is better to run EEMS several times, randomly initializing
#' the MCMC chain each time. In other words, simulate several realizations of the Markov chain
#' and let each realization start from a different state in the parameter space (by using a
#' different random seed).
#'
#' Detail about the within-deme and between-deme components of genetic dissimilarity: Let
#' \code{D(a,b)} be the dissimilarity between one individual from deme \code{a} and another
#' individual from deme \code{b}. Then the within-deme component for \code{a} and \code{b}
#' is simply \code{D(a,a)} and \code{D(b, b)}, respectively. The between-deme component is
#' \code{D(a,b) - [D(a,a) + D(b,b)] / 2} and it represents dissimilarity that is due to the spatial
#' structure of the population and is not a consequence of the local diversity in the two demes.
#' @param mcmcpath A vector of EEMS output directories, for the same dataset. Warning: There is
#' minimal checking that the given  directories are for the same dataset.
#' @param plotpath The full path and the file name for the graphics to be generated.
#' @param longlat A logical value indicating whether the coordinates are given as pairs
#' (longitude, latitude) or (latitude, longitude).
#' @param plot.width The width of the graphics region for the two rate contour plots, in inches.
#' The default value is 10.
#' @param plot.height The height of the graphics region, in inches. The default value is 10.
#' @param out.png A logical value which, if set,  forces output graphics to be generated as PNGs
#' (if `TRUE`) or PDFs (if `FALSE`). If left unset, the format depends on the nature of the plot. 
#' @param res Resolution, in dots per inch; used only for PNG images.
#' The default is 600.
#' @param xpd A logical value indicating whether to clip plotting to the figure region
#' (\code{xpd} = TRUE, which is the default) or clip plotting to the plot region
#' (\code{xpd} = FALSE).
#' @param add.grid A logical value indicating whether to add the population grid or not.
#' @param col.grid The color of the population grid. Defaults to \code{gray80}.
#' @param lwd.grid The line width of the population grid. Defaults to 1.
#' @param add.outline A logical value indicating whether to add the habitat outline or not.
#' @param col.outline The color of the habitat outline. Defaults to \code{white}.
#' @param lwd.outline The line width of the habitat outline. Defaults to 2.
#' @param add.demes A logical value indicating whether to add the observed demes or not.
#' @param col.demes The color of the demes. Defaults to \code{black}.
#' @param pch.demes The symbol, specified as an integer, or the character to be used for
#' plotting the demes. Defaults to 19.
#' @param min.cex.demes The minimum size of the deme symbol/character.
#' @param max.cex.demes The maximum size of the deme symbol/character. Defaults to 1 and 3,
#' respectively. If \code{max.cex.demes} > \code{min.cex.demes}, then demes with more samples
#' also have bigger size: the deme with the fewest samples has size \code{min.cex.demes} and
#' the deme with the most samples has size \code{max.cex.demes}.
#' @param projection.in The input cartographic projection, specified as a PROJ.4 string.
#' @param projection.out The output cartographic projection, specified as a PROJ.4 string.
#' @param add.map A logical value indicating whether to add a high-resolution geographic map.
#' Requires the \code{rworldmap} and \code{rworldxtra} packages. It also requires that
#' \code{projection.in} is specified.
#' @param col.map The color of the geographic map. Default is \code{gray60}.
#' @param lwd.map The line width of the geographic map. Defaults to 2.
#' @param eems.colors The EEMS color scheme as a vector of colors, ordered from low to high.
#' Defaults to a DarkOrange to Blue divergent palette with six orange shades, white in the middle,
#' six blue shades. Acknowledgement: The default color scheme is adapted from the \code{dichromat}
#' package.
#' @param m.colscale A fixed range for log10-transformed migration rates. If the estimated rates
#' fall outside the specified range, then the color scale is ignored. By default, no range is
#' specified for either type of rates.
#' @param q.colscale A fixed range for log10-transformed diversity rates.
#' @param add.colbar A logical value indicating whether to add the color bar (the key that shows
#' how colors map to rates) to the right of the plot. Defaults to TRUE.
#' @param remove.singletons Remove demes with a single observation from the diagnostic scatter
#' plots. Defaults to TRUE.
#' @param add.abline Add the line \code{y = x} to the diagnostic scatter plots of observed vs
#' fitted genetic dissimilarities.
#' @param add.r.squared Add the R squared coefficient to the diagnostic scatter plots of observed
#' vs fitted genetic dissimilarities.
#' @param add.title A logical value indicating whether to add the main title in the contour plots.
#' Defaults to TRUE.
#' @param prob.levels A vector of probabilities for plotting the posterior probability contours of
#' \code{P(m > 0 | diffs)} and \code{P(m < 0 | diffs)}. Defaults to \code{c(0.9, 0.95)}.
#' @param m.plot.xy Statements which add graphical elements (e.g. points) on top of the migration
#' sufrace.
#' @param q.plot.xy Statements which add graphical elements (e.g. points) on top of the diversity
#' surface.
#' @param xy.coords Additional coordinates at which to estimate the migration and diversity rates.
#' @returns None 
#' @references Light A and Bartlein PJ (2004). The End of the Rainbow? Color Schemes for Improved
#' Data Graphics. EOS Transactions of the American Geophysical Union, 85(40), 385.
#' @examples
#' # Use the provided example or supply the path to your own EEMS run.
#' extdata_path <- system.file("extdata", package = "reems")
#' eems_results <- file.path(extdata_path, "EEMS-example")
#' # Create a temporary output directory for the sake of the example
#' outdir <- file.path(tempdir(), "plot_out")
#' dir.create(outdir, showWarnings = FALSE) 
#' name_figures <- file.path(outdir, "EEMS-example")
#'
#' # Produce the set of EEMS figures, with default values for all optional parameters.
#' eems.plots(
#'   mcmcpath = eems_results,
#'   plotpath = paste0(name_figures, "-default"),
#'   longlat = TRUE,
#'   out.png = FALSE
#' )
#' 
#' # Delete the temporary output directory to tidy up. 
#' unlink(outdir, recursive = TRUE, force = TRUE)
#' @seealso \code{\link{eems.voronoi.samples}, \link{eems.posterior.draws}, \link{eems.population.grid}}
#' @export

eems.plots <- function(mcmcpath,
                       plotpath,
                       longlat,
                       plot.width = 10,
                       plot.height = 10,
                       out.png = NULL,
                       res = 600,
                       xpd = TRUE,
                       add.grid = FALSE,
                       col.grid = "gray80",
                       lwd.grid = 1,
                       add.demes = FALSE,
                       col.demes = "black",
                       pch.demes = 19,
                       min.cex.demes = 1,
                       max.cex.demes = 3,
                       add.outline = FALSE,
                       col.outline = "gray90",
                       lwd.outline = 2,
                       projection.in = NULL,
                       projection.out = NULL,
                       add.map = FALSE,
                       col.map = "gray60",
                       lwd.map = 2,
                       eems.colors = NULL,
                       prob.levels = c(0.9, 0.95),
                       add.colbar = TRUE,
                       m.colscale = NULL,
                       q.colscale = NULL,
                       remove.singletons = TRUE,
                       add.abline = FALSE,
                       add.r.squared = FALSE,
                       add.title = TRUE,
                       m.plot.xy = NULL,
                       q.plot.xy = NULL,
                       xy.coords = NULL) {
  plot.params <- list(
    eems.colors = eems.colors, m.colscale = m.colscale, q.colscale = q.colscale,
    add.map = add.map, add.grid = add.grid, add.outline = add.outline, add.demes = add.demes,
    col.map = col.map, col.grid = col.grid, col.outline = col.outline, col.demes = col.demes,
    lwd.map = lwd.map, lwd.grid = lwd.grid, lwd.outline = lwd.outline, pch.demes = pch.demes,
    min.cex.demes = min.cex.demes, proj.in = projection.in, add.colbar = add.colbar,
    max.cex.demes = max.cex.demes, proj.out = projection.out, add.title = add.title,
    prob.levels = prob.levels
  )
  plot.params <- check.plot.params(plot.params)

  # A vector of EEMS output directories, for the same dataset.
  # Assume that if eemsrun.txt exists, then all EEMS output files exist.
  mcmcpath <- mcmcpath[file.exists(file.path(mcmcpath, "eemsrun.txt"))]
  if (!length(mcmcpath)) {
    stop("Please provide at least one existing EEMS output directory, mcmcpath")
  }

  dimns <- read.dimns(mcmcpath[1], longlat, coord = xy.coords)

  save.params <- list(height = plot.height, width = plot.width, res = res, out.png = out.png)

  message("Processing the following EEMS output directories :")
  message(mcmcpath)

  
  # Plot filled contour of estimated effective migration rates
  mrates.raster <- plot_with_graphics_device(
      paste0(plotpath, "-mrates"),
      save.params,
      average.eems.contours(
          mcmcpath, dimns, longlat, plot.params,
          is.mrates = TRUE, plot.xy = m.plot.xy
      ),
      def.out = TRUE,
      par.args = list(las = 1, font.main = 1, xpd = xpd)
  )
  xym.values <- mrates.raster$xyz.values
  colnames(xym.values) <- c("x", "y", "m")

   # Plot filled contour of estimated effective diversity rates
  qrates.raster <- plot_with_graphics_device(
      paste0(plotpath, "-qrates"),
      save.params, 
      average.eems.contours(
          mcmcpath, dimns, longlat, plot.params,
          is.mrates = FALSE, plot.xy = q.plot.xy
      ),
      def.out = TRUE,
      par.args = list(las = 1, font.main = 1, xpd = xpd)
  )
  xyq.values <- qrates.raster$xyz.values
  colnames(xyq.values) <- c("x", "y", "q")

  if (!add.colbar) {
    if (out.png) {
      save.params$height <- 6
      save.params$width <- 1.5
    } else {
      save.params$height <- 12
      save.params$width <- 3
    }
    plot_with_graphics_device(
        paste0(plotpath, "-mkey"),
        save.params,
        myfilled.legend(
            levels = mrates.raster$eems.levels,
            col = mrates.raster$eems.colors,
            key.axes = axis(4, tick = FALSE, hadj = 1, line = 4, cex.axis = 2),
            key.title = mtext(expression(paste(log, "(", italic(m), ")", sep = "")),
                              side = 3, cex = 2.5, line = 1.5, font = 1
                              )
        ),
        def.out = TRUE,
        par.args = list(las = 1, font.main = 1, xpd = xpd, mar = c(0, 1, 5, 8))
    )
    plot_with_graphics_device(
        paste0(plotpath, "-qkey"),
        save.params,
        myfilled.legend(
            levels = qrates.raster$eems.levels,
            col = qrates.raster$eems.colors,
            key.axes = axis(4, tick = FALSE, hadj = 1, line = 6, cex.axis = 2),
            key.title = mtext(expression(paste(log, "(", italic(q), ")", sep = "")),
                              side = 3, cex = 2.5, line = 1.5, font = 1
                              )
        ),
        def.out = TRUE,
        par.args = list(las = 1, font.main = 1, xpd = xpd, mar = c(0, 1, 5, 8))
    )
  }

  save.params$height <- 6
  save.params$width <- 6.5
  # Plot trace plot of posterior probability to check convergence
  plot_with_graphics_device(
      paste0(plotpath, "-pilogl"),
      save.params,
      plot.logposterior(mcmcpath),
      def.out = FALSE,
      par.args = list(las = 0, font.main = 1, mar = c(5, 5, 4, 5) + 0.1, xpd = TRUE)
  )
  

  # Plot scatter plots of observed vs fitted genetic differences
  dist.points <- plot_with_graphics_device(
      paste0(plotpath, "-rdist"),
      save.params,
      dist.scatterplot(
          mcmcpath, longlat, plot.params,
          remove.singletons = remove.singletons,
          add.abline = add.abline, add.r.squared = add.r.squared
      ),
      def.out = FALSE,
      par.args = list(las = 1, font.main = 1, mar = c(5, 5, 4, 2) + 0.1)
  )

  B.component <- dist.points$B.component
  W.component <- dist.points$W.component
  G.component <- dist.points$G.component
  save(B.component, W.component, G.component, xym.values, xyq.values,
    file = paste0(plotpath, "-rdist.RData")
  )
}

test_that("eems.plots() writes output under default conditions", {
    extdata_path <- system.file("extdata", package = "reems")
    eems_results <- file.path(extdata_path, "EEMS-example")
    outdir <- file.path(tempdir(), "plot_out")
    dir.create(outdir, showWarnings = FALSE) 
    name_figures <- file.path(outdir, "EEMS-example")
    
    eems.plots(
        mcmcpath = eems_results,
        plotpath = paste0(name_figures, "-default"),
        longlat = TRUE
    )
    
    files <- list.files(outdir, full.names = TRUE)
    expect_length(files, 9)
    expect_true(all(file.info(files)$size > 0))
    unlink(outdir, recursive = TRUE, force = TRUE)
})

test_that("eems.plots() writes output when called with map overlay", {
    extdata_path <- system.file("extdata", package = "reems")
    eems_results <- file.path(extdata_path, "EEMS-example")
    outdir <- file.path(tempdir(), "plot_out")
    dir.create(outdir, showWarnings = FALSE) 
    name_figures <- file.path(outdir, "EEMS-example")
    
    projection_none <- "+proj=longlat +datum=WGS84"
    projection_mercator <- "+proj=merc +datum=WGS84"
    
    map_world <- rworldmap::getMap()
    map_africa <- map_world[which(map_world@data$continent == "Africa"), ]
    map_africa <- sp::spTransform(map_africa,  sp::CRS(projection_mercator))
    eems.plots(
        mcmcpath = eems_results,
        plotpath =   paste0(name_figures, "-with-map"),
        longlat = TRUE,
        projection.in = projection_none,
        projection.out = projection_mercator,
        m.plot.xy = {
            plot(map_africa, col = NA, add = TRUE)
        },
        q.plot.xy = {
            plot(map_africa, col = NA, add = TRUE)
        }
    )

    files <- list.files(outdir, full.names = TRUE)
    expect_length(files, 9)
    expect_true(all(file.info(files)$size > 0))
    unlink(outdir, recursive = TRUE, force = TRUE)    
})

test_that("eems.plots() writes reasonable objects to .RData file", {
    extdata_path <- system.file("extdata", package = "reems")
    eems_results <- file.path(extdata_path, "EEMS-example")
    outdir <- file.path(tempdir(), "plot_out")
    dir.create(outdir, showWarnings = FALSE) 
    name_figures <- file.path(outdir, "EEMS-example")
    
    xy.coords <- matrix(c(
        13.70, 3.20,
        37.10, -7.20,
        36.10, -4.10,
        34.58, -5.67
    ), ncol = 2, byrow = TRUE)
    
    eems.plots(
        mcmcpath = eems_results,
        plotpath = paste0(name_figures,"-with-coords"),
        longlat = TRUE,
        xy.coords = xy.coords
    )
    
    files <- list.files(outdir, full.names = TRUE)
    expect_length(files, 9)
    expect_true(all(file.info(files)$size > 0))

    file_path <- paste0(name_figures,"-with-coords", "-rdist.RData")
    temp_env <- new.env()
    load(file_path, envir = temp_env)
    expect_equal(nrow(temp_env$B.component),1485)
    expect_equal(ncol(temp_env$B.component),8)
    expect_equal(nrow(temp_env$W.component),55)
    expect_equal(ncol(temp_env$W.component),6)
    expect_equal(nrow(temp_env$G.component),1485)
    expect_equal(ncol(temp_env$G.component),8)
    expect_equal(nrow(temp_env$xym.values),4)
    expect_equal(ncol(temp_env$xym.values),3)
    expect_equal(nrow(temp_env$xyq.values),4)
    expect_equal(ncol(temp_env$xyq.values),3)
})

#' Create a buffered habitat around a set of points.
#'
#' Creates a polygon a certain distance from each of a set of points.
#' It is used for creating a default habitat outline for [eems()].
#' @param coords A two-column matrix of point coordinates (longitude, latitude).
#' @param d Rough buffer distance. Not necessarily kept completely to keep the polygon simple.
#' Defaults to 0.2 of the length of the hull of the `coords`. 
#' @returns A two-column matrix of coordinates with the outer boundary points of the containing polygon. 
#' @examples
#' #  Create a buffer around the example habitat
#' outer <- outer.buffered(ex.coords)
#' @export
outer.buffered <- function(coords, d = NULL) {
    pts <- st_as_sf(
      data.frame(x = coords[,1], y = coords[,2]),
    coords = c("x", "y")
  )    
    hull <- pts |>
        st_union() |>
        st_convex_hull()
    if (is.null(d)) {
        d = 0.2 * st_length(hull)
    }
    buffered <- st_buffer(hull, dist = d)
    simple_buffered <- st_simplify(buffered, dTolerance = 0.5*d)
    boundary <- st_cast(simple_buffered, "POLYGON")
    xy <- st_coordinates(boundary)[, 1:2]
    xy
    
}

geno2diffs <- function(Geno) {
  nIndiv <- nrow(Geno)
  nSites <- ncol(Geno)
  Miss <- is.na(Geno)
  ## Impute NAs with the column means (= twice the allele frequencies)
  Mean <- matrix(colMeans(Geno, na.rm = TRUE), ## a row of means
		 nrow = nIndiv, ncol = nSites, byrow = TRUE) ## a matrix with nIndiv identical rows of means
  GenoImputed <- Geno
  Mean[Miss == 0] <- 0 ## Set the means that correspond to observed genotypes to 0
  GenoImputed[Miss == 1] <- 0 ## Set the missing genotypes to 0 (used to be NA) 
  GenoImputed <- GenoImputed + Mean
  ## Compute similarities
  Sim <- GenoImputed %*% t(GenoImputed) / nSites
  SelfSim <- diag(Sim) ## self-similarities
  vector1s <- rep(1, nIndiv) ## vector of 1s
  ## This chunk generates a `diffs` matrix
  Diffs <- SelfSim %*% t(vector1s) + vector1s %*% t(SelfSim) - 2 * Sim
  Diffs
}

write_for_eems <- function(matrix, filename) { 
    write.table(matrix, filename, 
                col.names = FALSE, row.names = FALSE, quote = FALSE)
}

myheatmap <- function(x, col = NULL, colscale = NULL) {
  if (!is.matrix(x)) x <- as.matrix(x)
  r <- nrow(x)
  c <- ncol(x)
  if (c == 1) {
    x <- matrix(rev(x[, 1]), nrow = 1)
  } else if (r == 1) {
    x <- matrix(rev(x[1, ]), ncol = 1)
  } else {
    x <- t(x)[rev(seq(r)), ]
  }
  if (is.null(colscale)) {
    colscale <- range(x, na.rm = TRUE)
  }
  palette <- mypalette(colors = col, colscale = colscale)
  image(x, col = palette$colors, breaks = palette$levels, axes = FALSE)
  palette
}

heatmap.resid <- function(datapath, mcmcpath) {
  mcmcpath <- mcmcpath
  nchains <- length(mcmcpath)
  message("Heatmap of n-by-n matrix of residuals (observed - fitted):")
  Diffs <- as.matrix(read.table(paste0(datapath, ".diffs"), header = FALSE))
  nIndiv <- nrow(Diffs)
  nChains <- 0
  Delta <- matrix(0, nIndiv, nIndiv)
  for (path in mcmcpath) {
    if (
      file.exists(file.path(path, "rdistJtDobsJ.txt")) &&
        file.exists(file.path(path, "rdistJtDhatJ.txt")) &&
        file.exists(file.path(path, "rdistoDemes.txt"))
    ) {
      message(path)
      nChains <- nChains + 1
      ipmap <- scan(file.path(path, "ipmap.txt"), what = numeric(), quiet = TRUE)
      Sizes <- as.numeric(table(ipmap))
      n <- length(ipmap) # number of samples
      o <- length(Sizes) # number of observed demes
      J <- Matrix::spMatrix(n, o, i = seq(n), j = ipmap, x = rep(1, n)) # indicator matrix
      J <- as.matrix(J)
      JtDhatJ <- as.matrix(read.table(file.path(path, "rdistJtDhatJ.txt"),
        header = FALSE
      ))
      Delta <- Delta + J %*% JtDhatJ %*% t(J)
    }
  }
  if (nchains == 0) {
    return(NULL)
  }
  Delta <- Delta / nchains
  Delta <- Delta - diag(diag(Delta))
  resid <- Diffs - Delta
  diag(resid) <- NA # The diagonal entries would always be zero
  resid
}

#' A function to plot a heatmap of the residual pairwise dissimilarities, abs(observed - fitted)
#'
#' Given a set of EEMS output directories, this function generates a heatmap of the n-by-n matrix
#' of residual dissimilarities between pairs of individuals. The residuals are the differences
#' between the observed and the fitted dissimilarities.
#' The function also saves the residual matrix to a file called \code{plotpath-eems-resid.RData}.
#' In both the residual matrix and the corresponding heat map, individuals are are in the same
#' order as in the input files \code{datapath.coord} and \code{datapath.diffs}. 
#' @param datapath The full path and the file name of the input Diffs matrix, which is not copied
#' by \code{runeems} to the output directory.
#' @param heatmap.cols The heatmap color palette as a vector of colors, ordered from low to high.
#' Defaults to the "Reds" divergent palette in the RColorBrewer package.
#' @param heatmap.colscale A fixed range for the heatmap colors. The default is NULL, so the color
#' space is the observed range of the residuals.
#' @param out.png A logical value which, if set,  forces output graphics to be generated as PNGs
#' (if `TRUE`) or PDFs (if `FALSE`). Defaults to `TRUE`.
#' @inheritParams eems.plots
#' @returns None  
#' @examples
#' # Use the provided example or supply the path to your own EEMS run
#' extdata_path <- system.file("extdata", package = "reems")
#' eems_dataset <- file.path(extdata_path, "EEMS-barrier")
#' eems_results <- file.path(extdata_path, "EEMS-barrier")
#' # Create a temporary output directory for this example 
#' outdir <- file.path(tempdir(), "path_out")
#' dir.create(outdir, showWarnings = FALSE)
#' name_figures <- file.path(outdir, "EEMS-barrier")
#'
#' eems.resid.heatmap(
#'   datapath = eems_dataset,
#'   mcmcpath = eems_results,
#'   plotpath = name_figures,
#'   heatmap.cols = c("gray99", "red"),
#'   out.png = FALSE
#' )
#' # Delete the output directory to tidy up. 
#' unlink(outdir, recursive = TRUE, force = TRUE)
#' @seealso \code{\link{eems.plots}, \link{eems.voronoi.samples}, \link{eems.posterior.draws},
#' \link{eems.population.grid}}
#' @noRd
eems.resid.heatmap <- function(datapath,
                               mcmcpath,
                               plotpath,
                               plot.width = 7,
                               plot.height = 7,
                               out.png = TRUE,
                               res = 600,
                               heatmap.cols = NULL,
                               heatmap.colscale = NULL) {
  load.required.package(
    package = "Matrix",
    required.by = "eems.resid.heatmap"
  )

  save.params <- list(height = plot.height, width = plot.width, res = res, out.png = out.png)

  # A vector of EEMS output directories, for the same dataset.
  # Assume that if eemsrun.txt exists, then all EEMS output files exist.
  mcmcpath <- mcmcpath[file.exists(file.path(mcmcpath, "eemsrun.txt"))]
  if (!length(mcmcpath)) {
    stop("Please provide one existing EEMS output directory, mcmcpath")
  }

  message("Processing the following EEMS output directories :")
  message(mcmcpath)

  eems.resid <- heatmap.resid(datapath, mcmcpath)
  save(eems.resid, file = paste0(plotpath, "-eems-resid.RData"))

  key <- plot_with_graphics_device(
      paste0(plotpath, "-eems-resid"),
      save.params,
      myheatmap(abs(eems.resid), col = heatmap.cols, colscale = heatmap.colscale),
      par.args = list(las = 1, font.main = 1, mar = c(0, 0, 0, 0) + 0.1)
  )


  if (out.png) {
    save.params$height <- 6
    save.params$width <- 1.5
  } else {
    save.params$height <- 12
    save.params$width <- 3
  }

  plot_with_graphics_device(
      paste0(plotpath, "-eems-resid-key"),
      save.params,
      myfilled.legend(
          levels = key$levels, col = key$colors,
          key.axes = axis(4, tick = FALSE, hadj = 1, line = 4, cex.axis = 2),
          key.title = mtext("abs(r)", side = 3, cex = 2.5, line = 1.5, font = 1)
      ),
      par.args = list(las = 1, font.main = 1, mar = c(1, 1, 5, 8))
  )
  invisible(NULL)
}

#' @useDynLib reems
#' @import Rcpp 
#' @import raster sf sp
#' @import graphics grDevices
#' @importFrom stats lm na.omit
#' @importFrom utils read.table write.table capture.output
NULL

load.required.package <- function(package, required.by) {
  if (!requireNamespace(package, quietly = TRUE)) {
    stop(paste0(
      "'", required.by, "' requires the '", package, "' package. ",
      "Please install '", package, "' first."
    ))
  } else {
    message(paste0("Loading ", package, " (required by ", required.by, ")"))
  }
}

# This function is mainly for testing purposes and will create on Voronoi diagram
# for each saved MCMC iteration
voronoi.diagram <- function(mcmcpath, dimns, longlat, plot.params,
                            post.draws = 1, is.mrates = TRUE) {
  message("Plotting Voronoi tessellation of estimated effective rates")
  if (is.mrates) {
    log_scale <- plot.params$m.log_scale
  } else {
    log_scale <- plot.params$q.log_scale
  }
  voronoi <- read.voronoi(mcmcpath, longlat, is.mrates, log_scale)
  rates <- voronoi$rates
  tiles <- voronoi$tiles
  xseed <- voronoi$xseed
  yseed <- voronoi$yseed

  # Choose one saved posterior draw at random
  niters <- length(tiles)
  riter <- sample(seq(niters), 1)
  message(mcmcpath)
  message("Draw ", riter, " (out of ", niters, ")")

  eems.colors <- plot.params$eems.colors
  num.levels <- length(eems.colors)
  if (is.mrates) {
    eems.levels <- eems.colscale(rates, num.levels, plot.params$m.colscale)
  } else {
    eems.levels <- eems.colscale(rates, num.levels, plot.params$q.colscale)
  }
  n.levels <- length(eems.levels)
  max.levels <- max(eems.levels)
  min.levels <- min(eems.levels)
  # Jump over stored parameters for draws 1 to (riter - 1)
  skip <- sum(tiles[riter:1][-1])
  now.tiles <- tiles[riter]
  now.rates <- rates[(skip + 1):(skip + now.tiles)]
  now.xseed <- xseed[(skip + 1):(skip + now.tiles)]
  now.yseed <- yseed[(skip + 1):(skip + now.tiles)]
  # Standardize the log-transformed rates, without taking into account
  # the relative size of the tiles (this is hard to do without a grid)
  now.rates <- now.rates - mean(now.rates)
  now.rates <- ifelse(now.rates > max.levels, max.levels, now.rates)
  now.rates <- ifelse(now.rates < min.levels, min.levels, now.rates)
  plot(0, 0,
    type = "n", xlim = dimns$xlim, ylim = dimns$ylim, asp = 1,
    axes = FALSE, xlab = "", ylab = "", main = ""
  )
  if (now.tiles == 1) {
    # There is only one tile
    tile.color <- eems.colors[round(n.levels / 2)]
    polygon(dimns$xlim, dimns$ylim, col = tile.color, border = "white")
  } else {
    # Plot each tile in turn (as a polygon)
    Voronoi <- deldir::deldir(now.xseed, now.yseed, rw = c(dimns$xlim, dimns$ylim))
    tilelist <- deldir::tile.list(Voronoi)
    for (c in 1:now.tiles) {
      tile.color <- eems.colors[findInterval(now.rates[c], eems.levels, all.inside = TRUE)]
      polygon(tilelist[[c]]$x, tilelist[[c]]$y, col = tile.color, border = "white")
    }
    filled.contour.graph(mcmcpath, longlat, plot.params)
  }
  if (plot.params$add.seeds) {
    points(now.xseed, now.yseed,
      pch = plot.params$pch.seeds, cex = plot.params$cex.seeds,
      col = plot.params$col.seeds, lwd = 3
    )
  }
  list(colors = eems.colors, levels = eems.levels)
}

#' A function to plot raw Voronoi diagrams of effective migration and diversity rates
#'
#' Given a set of EEMS output directories, this function takes random draws from the posterior
#' distribution of the migration and diversity rate parameters. Each draw is visualized as two
#' raw Voronoi diagrams; the migration diagram is saved to a file ending in \code{mvoronoiXX},
#' the diversity diagram is saved to a file ending in \code{qvoronoiXX} where \code{XX} is
#' a numeric id. Specify the number of times to draw from the posterior with the argument
#' \code{post.draws}. If \code{post.draws = 10}, then \code{eems.voronoi.samples} will generate
#' plots with id \code{XX = 1} to \code{XX = 10}.
#' This function differs from [eems.posterior.draws()] by displaying raw, unsmoothed Voronoi diagrams. 
#'
#' Note about the implementation: \code{eems.voronoi.samples} samples randomly from the posterior
#' draws saved during the execution of EEMS, after burn-in and thinning.
#' @param post.draws Number of times to sample from the posterior. The default is 1.
#' @param add.seeds A logical value indicating whether to add the Voronoi seeds or not.
#' @param col.seeds The color of the Voronoi seeds. Defaults to \code{green}.
#' @param pch.seeds The symbol, specified as an integer, or the character to be used for
#' plotting the Voronoi seeds. Defaults to 4.
#' @param cex.seeds The size of the symbol/character used for plotting the Voronoi seeds.
#' Defaults to 1.
#' @param cex.demes The size of the symbol/character used for plotting observed demes.
#' Defaults to 1.
#' @param out.png A logical value which, if set,  forces output graphics to be generated as PNGs
#' (if `TRUE`) or PDFs (if `FALSE`). Defaults to `FALSE`.
#' @inheritParams eems.plots
#' @returns None  
#' @references Light A and Bartlein PJ (2004). The End of the Rainbow? Color Schemes for
#' Improved Data Graphics. EOS Transactions of the American Geophysical Union, 85(40), 385.
#' @examples
#' # Use the provided example or supply the path to your own EEMS run.
#' extdata_path <- system.file("extdata", package = "reems")
#' eems_results <- file.path(extdata_path, "EEMS-example")
#' # Create a temporary output directory for this example 
#' outdir <- file.path(tempdir(), "path_out")
#' dir.create(outdir, showWarnings = FALSE)
#' name_figures <- file.path(outdir, "eemsplot_out")
#'
#' # Plot a series of Voronoi diagrams for the EEMS model parameters:
#' # the effective migration rates (m) and the effective diversity rates (q).
#' eems.voronoi.samples(
#'   mcmcpath = eems_results,
#'   plotpath = paste0(name_figures, "-voronoi-diagrams"),
#'   longlat = TRUE,
#'   post.draws = 2,
#'   out.png = FALSE
#' )
#' # Delete the output directory to tidy up. 
#' unlink(outdir, recursive = TRUE, force = TRUE)
#' @seealso \code{\link{eems.posterior.draws}}
#' @export

eems.voronoi.samples <- function(mcmcpath,
                                 plotpath,
                                 longlat,
                                 post.draws = 1,
                                 plot.width = 10,
                                 plot.height = 10,
                                 out.png = FALSE,
                                 res = 600,
                                 add.grid = FALSE,
                                 col.grid = "gray80",
                                 lwd.grid = 1,
                                 add.outline = TRUE,
                                 col.outline = "gray80",
                                 lwd.outline = 2,
                                 add.demes = FALSE,
                                 col.demes = "gray80",
                                 pch.demes = 19,
                                 cex.demes = 1,
                                 add.seeds = TRUE,
                                 col.seeds = "#8AE234",
                                 pch.seeds = 4,
                                 cex.seeds = 1,
                                 eems.colors = NULL,
                                 m.colscale = NULL,
                                 q.colscale = NULL,
                                 add.title = FALSE) {
  load.required.package(package = "deldir", required.by = "eems.voronoi.samples")

  plot.params <- list(
    eems.colors = eems.colors, m.colscale = m.colscale, q.colscale = q.colscale,
    add.grid = add.grid, add.outline = add.outline, add.demes = add.demes, add.seeds = add.seeds,
    col.grid = col.grid, col.outline = col.outline, col.demes = col.demes, col.seeds = col.seeds,
    lwd.grid = lwd.grid, lwd.outline = lwd.outline, pch.demes = pch.demes, pch.seeds = pch.seeds,
    cex.seeds = cex.seeds, min.cex.demes = cex.demes, max.cex.demes = cex.demes,
    add.title = add.title
  )
  plot.params <- check.plot.params(plot.params)

  # A vector of EEMS output directories, for the same dataset.
  # Assume that if eemsrun.txt exists, then all EEMS output files exist.
  mcmcpath <- mcmcpath[file.exists(file.path(mcmcpath, "eemsrun.txt"))]
  if (!length(mcmcpath)) {
    stop("Please provide at least one existing EEMS output directory, mcmcpath")
  }

  dimns <- read.dimns(mcmcpath, longlat)
  save.params <- list(height = plot.height, width = plot.width, res = res, out.png = out.png)

  message("Processing the following EEMS output directory :")
  message(mcmcpath)

  plot.params$add.grid <- add.grid
  plot.params$all.demes <- FALSE
  plot.params$add.demes <- FALSE

  plot_with_graphics_device(
      paste0(plotpath, "-mvoronoi"),
      save.params,
      for (draw in seq(post.draws)) {
                                        # Choose one output directory at random
          voronoi.diagram(sample(mcmcpath, 1),
                          dimns, longlat, plot.params,
                          post.draws = post.draws, is.mrates = TRUE
                          )
      },
      par.args = list(las = 1, font.main = 1, mar = c(0, 0, 0, 0) + 0.1)
  )
  
  plot.params$add.grid <- FALSE
  plot.params$all.demes <- add.demes

  plot_with_graphics_device(
      paste0(plotpath, "-qvoronoi"),
      save.params,
      for (draw in seq(post.draws)) {
                                        # Choose one output directory at random
          voronoi.diagram(sample(mcmcpath, 1),
                          dimns, longlat, plot.params,
                          post.draws = post.draws, is.mrates = FALSE
                          )
      },
      par.args = list(las = 1, font.main = 1)
  )
invisible(NULL)  
}

test_that("eems.voronoi.samples writes output", {
    extdata_path <- system.file("extdata", package = "reems")
    eems_results <- file.path(extdata_path, "EEMS-example")
    outdir <- file.path(tempdir(), "path_out")
    dir.create(outdir, showWarnings = FALSE)
    name_figures <- file.path(outdir, "eemsplot_out")

    eems.voronoi.samples(
        mcmcpath = eems_results,
        plotpath = paste0(name_figures, "-voronoi-diagrams"),
        longlat = TRUE, post.draws = 2
)
    
    files <- list.files(outdir, full.names = TRUE)
    expect_length(files, 4)
    expect_true(all(file.info(files)$size > 0))
    unlink(outdir, recursive = TRUE, force = TRUE)
})

    

#' Run an EEMS analysis
#'
#' This function runs an EEMS analysis with given parameters, which include file paths for
#' retrieving the input files (datapath.coord, datapath.diffs and datapath.outer) and
#' writing the output files.
#' It is an exact replicate of the original EEMS command-line function. 
#'
#' @param seed The random seed. Defaults to the current system time in seconds. 
#' @param datapath Full path to a set of three files: datapath.coord, datapath.diffs and datapath.outer.
#' @param mcmcpath Full path to a filename prefix in an output directory with write permission. 
#' @param prevpath Full path to previous output directory, i.e., the mcmcpath in a previous EEMS run. Optional.
#' @param gridpath Full path to a set of two files: gridpath.demes and gridpath.edges. Optional.
#' @param nDemes Number of demes, roughly. EEMS constructs a regular triangular grid with circa nDemes vertices.
#' @param nIndiv Number of samples. Should match the size of the dissimilarity matrix in datapath.diffs.
#' @param nSites Number of SNPs used to compute the observed dissimilarity matrix in datapath.diffs.
#' @param diploid Logical value that indicates whether the species is diploid (TRUE) or haploid (FALSE). Defaults to TRUE.
#' @param distance Distance metric. Either `euclidean` or `greatcirc`, that is, great circle distance.
#' Defaults to 'euclidean'.
#' @param numMCMCIter Number of Markov Chain Monte Carlo iterations. 
#' @param numBurnIter Number of burn-in iterations to be discarded before sampling from posterior. 
#' @param numThinIter Number of thinning iterations to be discarded between sampling from posterior. 
#' @param mSeedsProposalS2 Variance of normal proposals to update the seeds of the migration tiles. Defaults to 0.01.
#' @param qSeedsProposalS2 Variance of normal proposals to update the seeds of the diversity tiles. Defaults to 0.1. 
#' @param mEffctProposalS2 Variance of normal proposals to update the log10 rates of the migration tiles. Defaults to 0.1.
#' @param qEffctProposalS2 Variance of normal proposals to update the log10 rates of the diversity tiles. Defaults to 0.001.
#' @param mrateMuProposalS2 Variance of normal proposals to update the overall mean migration rate, on the log10 scale.
#' Defaults to 0.01.
#' @param qVoronoiPr With probability qVoronoiPr, update diversity Voronoi;
#' with probability 1-qVoronoiPr, update migration Voronoi. Defaults to 0.25.
#' @param qrateShape Shape hyperparameter for the diversity rates variance, qrateS2 ~ invgamma(qrateShape, qrateScale).
#' Defaults to 0.001
#' @param mrateShape Shape hyperparameter for the migration rates variance, mrateS2 ~ invgamma(mrateShape, mrateScale).
#' Defaults to 0.001.
#' @param sigmaShape Shape hyperparameter for the scaling factor sigma^2 ~ invgamma(sigmaShape, sigmaScale).
#' Defaults to 0.001.
#' @param qrateScale Scale hyperparameter for the diversity rates variance, qrateS2 ~ invgamma(qrateShape, qrateScale).
#' Defaults to 1.0.
#' @param mrateScale Scale hyperparameter for the migration rates variance, mrateS2 ~ invgamma(mrateShape, mrateScale).
#' Defaults to 1.0.
#' @param sigmaScale Scale hyperparameter for the scaling factor sigma^2 ~ invgamma(sigmaShape, sigmaScale).
#' Defaults to 1.0.
#' @param negBiProb Success probability for the number of Voronoi tiles ~ negbinom(negBiSize, negBiProb). Defaults to 0.67.
#' @param negBiSize Size for the number of Voronoi tiles ~ negbinom(negBiSize, negBiProb). Defaults to 10.
#' @returns None
#' @references Petkova, D., Novembre, J. & Stephens, M.
#' Visualizing spatial population structure with estimated effective migration surfaces.
#' Nat Genet 48, 94–100 (2016). https://doi.org/10.1038/ng.3464
#' @examples
#' # We use example input from Petkova et al. (2016), found in the '/extdata' directory
#' data_path <- system.file("extdata", package = "reems")
#' input <- file.path(data_path, "barrier-schemeX-nIndiv300-nSites3000")
#' # The example puts the output in a temporary directory.
#' mcmcdir <- file.path(tempdir(), "eems_out")
#' dir.create(mcmcdir, showWarnings = FALSE)
#' 
#'
#' # Run an example EEMS analysis with a small number of iterations to ensure quick termination. 
#' 
#' eems.from.files(
#'   datapath = input,
#'   mcmcpath = mcmcdir,
#'   nDemes = 200,
#'   nIndiv = 300,
#'   nSites = 3000,
#'   diploid = FALSE,
#'   numMCMCIter = 200,
#'   numBurnIter = 100,
#'   numThinIter = 99,
#' )
#' # Delete the output directory to tidy up. 
#' unlink(mcmcdir, recursive = TRUE, force = TRUE)
#' @seealso \code{\link{eems.plots}, \link{eems}}
#' @export
eems.from.files <- function(
                            seed = unclass(Sys.time()),
                            datapath,
                            mcmcpath,
                            prevpath = "",
                            gridpath = "",
                            nDemes,
                            nIndiv,
                            nSites,
                            diploid = TRUE,
                            distance = "euclidean",
                            numMCMCIter,
                            numBurnIter,
                            numThinIter,
                            mSeedsProposalS2 = 0.01,
                            qSeedsProposalS2 = 0.1,
                            mEffctProposalS2 = 0.1,
                            qEffctProposalS2 = 0.001,
                            mrateMuProposalS2 = 0.01,
                            qVoronoiPr = 0.25,
                            qrateShape = 0.001,
                            mrateShape = 0.001,
                            sigmaShape = 0.001,                            
                            qrateScale = 1.0,
                            mrateScale = 1.0,
                            sigmaScale = 1.0,
                            negBiProb = 0.67,
                            negBiSize = 10) {
    testing = FALSE
    p <- as.list(environment())
    cpp_eems(p)
}

#' Run an EEMS analysis from data in R format
#'
#' This function runs an EEMS analysis with given parameters in the input format for `conStruct`,
#' which include R matrices for allele frequency data and coordinates.  
#' In addition to taking R objects as inputs,
#' it also provides additional default values over the original `EEMS` command-line utility.
#' Furthermore, it allows for running more than one chain, either in parallel or sequentially,
#' and returns a vector of output directories as input to [eems.plots()].  
#'
#' @param coords The coordinate matrix, a two-column matrix with a line for every sample.
#' @param freqs An allele frequency  matrix in `conStruct` format, with a column for every site and a row for every sample.
#' Choosing one main allele, it tabulates the average genotypes of the sample at that locus.  
#' @param outer Matrix of coordinates of the outer polygon.
#' Defaults to a polygon larger than the convex hull of the supplied coordinates to mitigate EEMS boundary effects,
#' which is created by [outer.buffered()].
#' @param buffer The rough distance of the boundary from the convex hull. Defaults to 0.2 times the length of the hull.
#' Unused if `outer` is specified.
#' @param nDemes Number of demes, roughly. EEMS constructs a regular triangular grid with circa nDemes vertices.
#' Defaults to 6 times the number of individuals, aiming for sufficient resolution. 
#' @param nChains Number of chains to be run. Defaults to 1.
#' @param parallel Logical values that indicates whether chains should be run in parallel. Defaults to FALSE.
#' @param nWorkers If `parallel` is TRUE, then the number of workers used is the minimum of `nChains` and `nWorkers`.
#' Defaults to the number of available cores.
#' @param distance Distance metric. Either `euclidean` or `greatcirc`, that is, great circle distance.
#' Defaults to 'greatcirc', which is more accurate at larger scales.
#' @param numMCMCIter Number of Markov Chain Monte Carlo iterations. Defaults to 2000000.
#' @param numBurnIter Number of burn-in iterations to be discarded before sampling from posterior. Defaults to 1000000.
#' @param numThinIter Number of thinning iterations to be discarded between sampling from posterior. Defaults to 9999.
#' @param demes Matrix of coordinates of demes; optional, and usually unnecessary. Only in conjunction with `edges`.
#' @param edges Matrix of custom input grid; optional, and usually unnecessary. Only in conjunction with `demes`. 
#' @inheritParams eems.from.files
#' @returns A vector of output directories which can be given as input to [eems.plots()]. 
#' @references Petkova, D., Novembre, J. & Stephens, M.
#' Visualizing spatial population structure with estimated effective migration surfaces.
#' Nat Genet 48, 94–100 (2016). https://doi.org/10.1038/ng.3464
#' @examples
#' # The example puts the output in a temporary directory.
#' mcmcdir <- file.path(tempdir(), "eems_out")
#' dir.create(mcmcdir, showWarnings = FALSE)
#' mcmcpath <- file.path(mcmcdir,"example")
#'
#' # Run an example EEMS analysis with a small number of iterations to ensure quick termination. 
#' 
#' eems(
#'   freqs = ex.freqs,
#'   coords = ex.coords,
#'   mcmcpath = mcmcpath,   
#'   numMCMCIter = 200,
#'   numBurnIter = 100,
#'   numThinIter = 99,
#' )
#' # Delete the output directory to tidy up. 
#' unlink(mcmcdir, recursive = TRUE, force = TRUE)
#' @seealso \code{\link{eems.plots}, \link{eems.from.files}}
#' @export
eems <- function(
                 seed = unclass(Sys.time()),
                 coords,
                 freqs,
                 outer = NULL,
                 buffer = NULL,
                 mcmcpath,
                 demes = NULL,
                 edges = NULL,
                 prevpath = "",
                 nChains = 1,
                 parallel = FALSE,
                 nWorkers = NULL,
                 nDemes = NULL,
                 diploid = TRUE,
                 distance = "greatcircle",
                 numMCMCIter = 2000000,
                 numBurnIter = 1000000,
                 numThinIter = 9999,
                 mSeedsProposalS2 = 0.01,
                 qSeedsProposalS2 = 0.1,
                 mEffctProposalS2 = 0.1,
                 qEffctProposalS2 = 0.001,
                 mrateMuProposalS2 = 0.01,
                 qVoronoiPr = 0.25,
                 qrateShape = 0.001,
                 mrateShape = 0.001,
                 sigmaShape = 0.001,                            
                 qrateScale = 1.0,
                 mrateScale = 1.0,
                 sigmaScale = 1.0,
                 negBiProb = 0.67,
                 negBiSize = 10) {
    if (is.null(outer)) {
        outer <- outer.buffered(coords,buffer) 
    }
    nIndiv <- nrow(freqs)
    nSites <- ncol(freqs)
    if (is.null(nDemes)) {
        nDemes <- 6*nIndiv 
    }
    data_path <- file.path(tempdir(), "data_path")
    dir.create(data_path, showWarnings = FALSE)
    on.exit(unlink(data_path, recursive = TRUE, force = TRUE), add = TRUE)
    datapath <- file.path(data_path,"in")
    coordfile <- paste0(datapath, ".coord")
    diffsfile <- paste0(datapath,".diffs")
    outerfile <- paste0(datapath,".outer")
    write_for_eems(coords,coordfile)
    write_for_eems(geno2diffs(freqs),diffsfile)
    write_for_eems(outer,outerfile)
    write_for_eems(coords,coordfile)    
    testing = FALSE

    if (!is.null(demes) && !is.null(edges)) {
        gridpath <- datapath
        demesfile <- paste0(gridpath, ".demes")
        edgesfile <- paste0(gridpath, ".edges")
        write_for_eems(demes, demesfile)
        write_for_eems(edges, edgesfile)
    }
    else {
        gridpath = ""
    }
    p <- as.list(environment())
    if (parallel){
        if(is.null(nWorkers)) {
            nWorkers <- min(nChains, future::availableCores(constraints = "connections-16"))
        }
        old_future_plan <- future::plan()
        on.exit(future::plan(old_future_plan), add = TRUE)
        future::plan(future::multisession, workers = nWorkers)
        paths <- future.apply::future_sapply(1:nChains, function(i) {
        pf <- p
        pf$mcmcpath <- paste0(p$mcmcpath, "-chain", i)
        dir.create(pf$mcmcpath, showWarnings = FALSE)
        pf$seed <- p$seed + i
        capture.output(cpp_eems(pf), file = file.path(pf$mcmcpath, "eems.log"))
        return(pf$mcmcpath)
        }, future.seed = TRUE)
        return(paths)
    }
    else {
        sapply(1:nChains, function(i) {
        pf <- p
        pf$mcmcpath <- paste0(p$mcmcpath, "-chain", i)
        pf$seed <- p$seed + i
        cpp_eems(pf)
        return(pf$mcmcpath)
        })
    }
}

test_that("sequential eems() writes output", {
    mcmcdir <- file.path(tempdir(), "eems_out")
    dir.create(mcmcdir, showWarnings = FALSE)
    mcmcpath <- file.path(mcmcdir,"example")
        
    dir <- eems(
        freqs = ex.freqs,
        coords = ex.coords,
        mcmcpath = mcmcpath,   
        numMCMCIter = 2000,
        numBurnIter = 1000,
        numThinIter = 99
    )
    
    
    files <- list.files(dir, full.names = TRUE)
    expect_length(files, 31)
    expect_true(all(file.info(files)$size > 0))
    unlink(mcmcdir, recursive = TRUE, force = TRUE)
})

test_that("parallel eems() writes output and logs", {
    mcmcdir <- file.path(tempdir(), "eems_out")
    dir.create(mcmcdir, showWarnings = FALSE)
    mcmcpath <- file.path(mcmcdir,"example")
        
    dirs <- eems(
        freqs = ex.freqs,
        coords = ex.coords,
        mcmcpath = mcmcpath,   
        numMCMCIter = 2000,
        numBurnIter = 1000,
        numThinIter = 99,
        parallel = TRUE,
        nWorkers = 2,
        nChains = 2
    )
    

    files <- list.files(dirs[[1]], full.names = TRUE)

    expect_length(files, 32)
    expect_true(all(file.info(files)$size > 0))

    files <- list.files(dirs[[2]], full.names = TRUE)

    expect_length(files, 32)
    expect_true(all(file.info(files)$size > 0))

    unlink(mcmcdir, recursive = TRUE, force = TRUE)
})

    

test_that("eems.from.files() writes output", {
    data_path <- system.file("extdata", package = "reems")
    input <- file.path(data_path, "barrier-schemeX-nIndiv300-nSites3000")
    mcmcdir <- file.path(tempdir(), "eems_out")
    dir.create(mcmcdir, showWarnings = FALSE)
    
eems.from.files(
    datapath = input,
    mcmcpath = mcmcdir,
    nDemes = 200,
    nIndiv = 300,
    nSites = 3000,
    diploid = FALSE,
    numMCMCIter = 2000,
    numBurnIter = 1000,
    numThinIter = 99,
    )
    
    files <- list.files(mcmcdir, full.names = TRUE)
    expect_length(files, 31)
    expect_true(all(file.info(files)$size > 0))
    unlink(mcmcdir, recursive = TRUE, force = TRUE)
})


    

#' Example allele frequency data from ConStruct
#'
#' @format ## `ex.freqs`
#' A matrix with 16 rows and 10000 columns.
#' The rows designate samples and the columns loci.
#' @source https://github.com/gbradburd/conStruct/tree/master/data
"ex.freqs"

#' Example coordinate data from ConStruct
#'
#' @format ## `ex.coords`
#' A matrix with 16 rows and 2 columns.
#' The rows designate samples and the two columns store the X and Y coordinate respectively. .
#' @source https://github.com/gbradburd/conStruct/tree/master/data
"ex.coords"

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cpp_eems <- function(p) {
    invisible(.Call('_reems_cpp_eems', PACKAGE = 'reems', p))
}

cpp_write_grid <- function(datapath, mcmcpath, nDemes) {
    invisible(.Call('_reems_cpp_write_grid', PACKAGE = 'reems', datapath, mcmcpath, nDemes))
}

tiles2contours_standardize <- function(tiles, rates, seeds, marks, distm) {
    .Call('_reems_tiles2contours_standardize', PACKAGE = 'reems', tiles, rates, seeds, marks, distm)
}

tiles2contours <- function(tiles, rates, seeds, marks, distm) {
    .Call('_reems_tiles2contours', PACKAGE = 'reems', tiles, rates, seeds, marks, distm)
}

# https://github.com/r-spatial/sf/issues/572
st_envelope <- function(x) {
  st_as_sfc(st_bbox(x))
}
st_is_ring <- function(x) {
  boundary <- st_polygonize(x)
  !st_is_empty(boundary)
}

# Redefine the 'filledContour' function from the 'raster' package
# since I don't like how the legend looks.
# For now, I have only changed how the legend/bar is plotted,
# the change itself is in the 'myfilled.contour' function
myfilledContour <- function(x, y = 1, maxpixels = 1e+05, add.key = TRUE, ...) {
  if (nlayers(x) > 1) {
    y <- min(max(1, y), nlayers(x))
    x <- raster(x, y)
  }
  x <- sampleRegular(x, maxpixels, asRaster = TRUE, useGDAL = TRUE)
  seq_cols <- seq_len(ncol(x))
  seq_rows <- rev(seq_len(nrow(x)))
  X <- xFromCol(x, seq_cols)
  Y <- yFromRow(x, seq_rows)
  Z <- t(matrix(getValues(x), ncol = x@ncols, byrow = TRUE)[seq_rows, ])
  myfilled.contour(
    x = X, y = Y, z = Z, add.key = add.key,
    axes = FALSE, frame.plot = FALSE, ...
  )
}

# I have only changed the legend/bar to remove the black border.
myfilled.contour <- function(x = seq(0, 1, length.out = nrow(z)),
                             y = seq(0, 1, length.out = ncol(z)), z,
                             xlim = range(x, finite = TRUE),
                             ylim = range(y, finite = TRUE),
                             zlim = range(z, finite = TRUE),
                             levels = pretty(zlim, nlevels), nlevels = 20,
                             color.palette = cm.colors, col = color.palette(length(levels) - 1),
                             plot.title, plot.axes, key.title, key.axes, asp = NA,
                             xaxs = "i", yaxs = "i", las = 1,
                             axes = TRUE, frame.plot = axes, add.title = FALSE, add.key = TRUE,
                             ...) {
  if (missing(z)) {
    if (!missing(x)) {
      if (is.list(x)) {
        z <- x$z
        y <- x$y
        x <- x$x
      } else {
        z <- x
        x <- seq.int(0, 1, length.out = nrow(z))
      }
    } else {
      stop("no 'z' matrix specified")
    }
  } else if (is.list(x)) {
    y <- x$y
    x <- x$x
  }
  if (any(diff(x) <= 0) || any(diff(y) <= 0)) {
    stop("increasing 'x' and 'y' values expected")
  }
  mar.orig <- (par.orig <- par(c("mar", "las", "mfrow")))$mar
  on.exit(par(par.orig))
  w <- (3 + mar.orig[2L]) * par("csi") * 2.54
  if (add.key) {
    layout(matrix(c(2, 1), ncol = 2L), widths = c(1, lcm(w)))
  }
  par(las = las)
  mar <- mar.orig
  mar[4L] <- mar[2L]
  mar[2L] <- 1
  par(mar = mar)
  if (add.key) {
    plot.new()
    par(plt = c(0, 0.3, 0.1, 0.9))
    # Change: add 'border = NA' and remove 'box()'
    plot.window(xlim = c(0, 1), ylim = range(levels), xaxs = "i", yaxs = "i")
    rect(0, levels[-length(levels)], 1, levels[-1L], col = col, border = NA)
    if (missing(key.axes)) {
      if (axes) {
        axis(4)
      }
    } else {
      key.axes
    }
    if (!missing(key.title)) {
      key.title
    }
  }
  mar <- mar.orig
  mar[4L] <- 1
  par(mar = mar)
  plot.new()
  # Change: make the plot region almost as big as the figure region
  #         but leave space for the main title
  if (add.title) {
    par(plt = c(0.02, 0.98, 0.02, 0.95))
  } else {
    par(plt = c(0.02, 0.98, 0.02, 0.98))
  }
  plot.window(xlim, ylim, "", asp = asp)
  .filled.contour(x, y, z, levels, col)
  if (missing(plot.axes)) {
    if (axes) {
      title(main = "", xlab = "", ylab = "")
      Axis(x, side = 1)
      Axis(y, side = 2)
    }
  } else {
    plot.axes
  }
  if (frame.plot) {
    box()
  }
  if (add.title) {
    if (missing(plot.title)) {
      title(...)
    } else {
      plot.title
    }
  }
  invisible()
}

myfilled.legend <-
  function(levels, color.palette = cm.colors,
           col = color.palette(length(levels) - 1), plot.title, plot.axes,
           key.title, key.axes, asp = NA, xaxs = "i", yaxs = "i", las = 1,
           axes = TRUE, frame.plot = axes, ...) {
    plot.new()
    plot.window(xlim = c(0, 1), ylim = range(levels), xaxs = "i", yaxs = "i")
    rect(0, levels[-length(levels)], 1, levels[-1L], col = col, border = NA)
    if (!missing(key.axes)) {
      key.axes
    } else {
      axis(4, tick = FALSE)
    }
    if (!missing(key.title)) {
      key.title
    }
  }

filled.contour.points <- function(mcmcpath, longlat, plot.params, highlight) {
  if (is.null(highlight)) {
    return(NULL)
  }
  if (is.null(highlight$index)) {
    message("Specify the indices of points to highlight")
    return(NULL)
  }
  if (is.null(highlight$col)) highlight$col <- "red"
  if (is.null(highlight$cex)) highlight$cex <- 1
  if (is.null(highlight$pch)) highlight$pch <- 4
  if (is.null(highlight$lwd)) highlight$lwd <- 2

  graph <- read.graph(mcmcpath, longlat)
  coord <- graph$demes[graph$ipmap, ]

  index2coord <- match(highlight$index, seq_len(nrow(coord)))
  index2highlight <- which(!is.na(index2coord))
  index2coord <- index2coord[index2highlight]

  # Check that there is at least one point to highlight
  if (!length(index2highlight)) {
    return(NULL)
  }

  coord <- coord[index2coord, ]
  coord <- matrix(coord, ncol = 2)
  # Transform the coordinates to a different projection if necessary
  if (!is.null(plot.params$proj.out)) {
    coord <- sp::SpatialPoints(coord, proj4string = CRS(plot.params$proj.in))
    coord <- sp::spTransform(coord, CRSobj = CRS(plot.params$proj.out))
  }
  points(coord,
    cex = highlight$cex[index2highlight],
    col = highlight$col[index2highlight],
    pch = highlight$pch[index2highlight],
    lwd = highlight$lwd[index2highlight]
  )
}

transform_rates <- function(dimns, tiles, rates, xseed, yseed, zero_mean) {
  # Bind together the points which make up the raster image
  # and the extra points provided by the user, if any
  marks <- rbind(dimns$marks, dimns$coord)
  coord_raster <- nrow(dimns$marks)
  coord_extra <- nrow(dimns$coord)

  # Compute the rates at all the points
  if (zero_mean) {
    rslts <- tiles2contours_standardize(
      tiles, rates, cbind(xseed, yseed),
      marks, dimns$dist.metric
    )
  } else {
    rslts <- tiles2contours(
      tiles, rates, cbind(xseed, yseed),
      marks, dimns$dist.metric
    )
  }

  Zvals <- matrix(rslts$zvals[seq_len(coord_raster)],
    nrow = dimns$nxmrks, ncol = dimns$nymrks
  )
  if (coord_extra > 0) {
    Zvals_extra <- rslts$zvals[seq_len(coord_extra) + coord_raster]
  } else {
    Zvals_extra <- NULL
  }
  if (zero_mean) {
    # Without normalizing the migration rates m to have mean 0,
    # it doesn't make sense to keep track of the number of times (m > 0) and (m < 0).
    PrGT0 <- matrix(rslts$prgt0[seq_len(coord_raster)],
      nrow = dimns$nxmrks, ncol = dimns$nymrks
    )
    PrLT0 <- matrix(rslts$prlt0[seq_len(coord_raster)],
      nrow = dimns$nxmrks, ncol = dimns$nymrks
    )
  } else {
    PrGT0 <- NULL
    PrLT0 <- NULL
  }
  list(
    Zvals = Zvals, PrGT0 = PrGT0, PrLT0 = PrLT0,
    Zvals_extra = Zvals_extra, niters = length(tiles)
  )
}

check.habitat.outer.is.valid <- function(mcmcpath, longlat) {
  graph <- read.graph(mcmcpath, longlat)
  outer <- graph$outer
  # Check that the geometry of the habitat outline is both Simple (gIsSimple)
  # and Closed (end points overlap).
  x <- outer[, 1]
  y <- outer[, 2]
  l <- st_as_sfc(paste("LINESTRING(", paste(paste(x, y), collapse = ", "), ")"))
  if (!st_is_ring(l)) {
    message(
      "Error:\n",
      "The habitat geometry is not a valid ring (a ring is both simple and closed)\n",
      "Let the habitat be the rectangle defined by (xmin, ymin) and (xmax, yxmax)\n",
      "where xmin, xmax = range(longitude) and ymin, ymax = range(latitude)."
    )
    xmin <- min(outer[, 1])
    xmax <- max(outer[, 1])
    ymin <- min(outer[, 2])
    ymax <- max(outer[, 2])
    outer <- cbind(c(xmin, xmax, xmax, xmin, xmin), c(ymin, ymin, ymax, ymax, ymin))
  }
  outer
}

filled.contour.map <- function(mcmcpath, longlat, plot.params) {
  if (!is.null(plot.params$proj.in) && plot.params$add.map) {
    map <- rworldmap::getMap(resolution = "high")
    map <- sp::spTransform(map, CRSobj = CRS(plot.params$proj.out))
    plot(map, col = NA, border = plot.params$col.map, lwd = plot.params$lwd.map, add = TRUE)
  }
}

filled.contour.outline <- function(mcmcpath, longlat, plot.params) {
  outer <- check.habitat.outer.is.valid(mcmcpath, longlat)
  boundary <- st_polygon(list(outer))
  if (!is.null(plot.params$proj.in)) {
    boundary <- st_sfc(boundary, crs = plot.params$proj.in)
    boundary <- st_transform(boundary, plot.params$proj.out)
  }
  # The filledContour fills a rectangular plot; now color the habitat exterior white.
  exterior <- st_difference(st_envelope(boundary), boundary)
  if (!is.null(exterior)) {
    plot(exterior, col = "white", border = "white", add = TRUE)
  }
  if (plot.params$add.outline) {
    plot(boundary,
      col = NA, border = plot.params$col.outline,
      lwd = plot.params$lwd.outline, add = TRUE
    )
  }
}

filled.contour.graph <- function(mcmcpath, longlat, plot.params) {
  graph <- read.graph(mcmcpath, longlat)
  if (plot.params$add.grid) {
    segments <- list()
    for (e in seq_len(nrow(graph$edges))) {
      segments[[e]] <- sp::Line(graph$demes[graph$edges[e, ], ])
    }
    segments <- sp::SpatialLines(list(Lines(segments, ID = "a")))
    if (!is.null(plot.params$proj.in)) {
      proj4string(segments) <- CRS(plot.params$proj.in)
      segments <- sp::spTransform(segments, CRSobj = CRS(plot.params$proj.out))
    }
    lines(segments, col = plot.params$col.grid, lwd = plot.params$lwd.grid)
  }
  if (plot.params$all.demes) {
    all.demes <- sp::SpatialPoints(matrix(graph$demes[graph$alpha, ],
      ncol = 2
    ))
    if (!is.null(plot.params$proj.in)) {
      proj4string(all.demes) <- CRS(plot.params$proj.in)
      all.demes <- sp::spTransform(all.demes, CRSobj = CRS(plot.params$proj.out))
    }
    points(all.demes,
      col = plot.params$col.demes,
      pch = plot.params$pch.demes, cex = plot.params$min.cex.demes
    )
  } else if (plot.params$add.demes) {
    # Make sure that observed.demes is a matrix even if there is one alpha
    observed.demes <- sp::SpatialPoints(matrix(graph$demes[graph$alpha, ],
      ncol = 2
    ))
    if (!is.null(plot.params$proj.in)) {
      proj4string(observed.demes) <- CRS(plot.params$proj.in)
      observed.demes <- sp::spTransform(observed.demes, CRSobj = CRS(plot.params$proj.out))
    }
    cex.points <- plot.params$min.cex.demes
    if (min(graph$sizes) < max(graph$sizes)) {
      cex.points <- cex.points +
        (plot.params$max.cex.demes - plot.params$min.cex.demes) *
          (graph$sizes - min(graph$sizes)) / (max(graph$sizes) - min(graph$sizes))
    }
    points(observed.demes,
      col = plot.params$col.demes,
      pch = plot.params$pch.demes, cex = cex.points
    )
  }
}

project_raster <- function(surface, proj.in, proj.out) {
  raster::projection(surface) <- CRS(proj.in)
  raster::projectRaster(surface, crs = CRS(proj.out))
}

points_to_raster <- function(points, dimns, params) {
  surface <- raster::raster(t(points),
    xmn = dimns$xlim[1], xmx = dimns$xlim[2],
    ymn = dimns$ylim[1], ymx = dimns$ylim[2]
  )
  surface <- raster::flip(surface, direction = "y")
  if (!is.null(params$proj.in)) {
    surface <- project_raster(surface, params$proj.in, params$proj.out)
  }
  surface
}

null.eems.contour <- function(mcmcpath, longlat, plot.params,
                              plot.xy = NULL, highlight.samples = NULL) {
  dimns <- read.dimns(mcmcpath, longlat)
  eems.colors <- plot.params$eems.colors
  num.levels <- length(eems.colors)
  eems.levels <- seq(from = -2.5, to = +2.5, length = num.levels + 1)
  main.title <- ""
  key.title <- ""
  Zeros <- matrix(0, nrow = dimns$nxmrks, ncol = dimns$nymrks)
  myfilledContour(
    points_to_raster(Zeros, dimns, plot.params),
    col = eems.colors, levels = eems.levels, asp = 1,
    add.key = plot.params$add.colbar,
    key.axes = axis(4, tick = FALSE, hadj = 1, line = 3, cex.axis = 1.5),
    key.title = mtext(key.title, side = 3, cex = 1.5, line = 1.5, font = 1),
    add.title = plot.params$add.title,
    plot.title = mtext(text = main.title, side = 3, line = 0, cex = 1.5),
    plot.axes = {
      filled.contour.outline(mcmcpath, longlat, plot.params)
      filled.contour.map(mcmcpath, longlat, plot.params)
      plot.xy
      filled.contour.graph(mcmcpath, longlat, plot.params)
      filled.contour.points(mcmcpath, longlat, plot.params, highlight.samples)
    }
  )
  list(eems.colors = eems.colors, eems.levels = eems.levels)
}

one.eems.contour <- function(mcmcpath, dimns, Zmean, longlat, plot.params, is.mrates,
                             plot.xy = NULL, highlight.samples = NULL) {
  eems.colors <- plot.params$eems.colors
  num.levels <- length(eems.colors)
  if (is.mrates) {
    eems.levels <- eems.colscale(Zmean, num.levels, plot.params$m.colscale)
    main.title <- "Posterior mean migration rates m (on the log10 scale)"
    key.title <- "log(m)"
  } else {
    eems.levels <- eems.colscale(Zmean, num.levels, plot.params$q.colscale)
    main.title <- "Posterior mean diversity rates q (on the log10 scale)"
    key.title <- "log(q)"
  }
  myfilledContour(
    points_to_raster(Zmean, dimns, plot.params),
    col = eems.colors, levels = eems.levels, asp = 1,
    add.key = plot.params$add.colbar,
    key.axes = axis(4, tick = FALSE, hadj = 1, line = 3, cex.axis = 1.5),
    key.title = mtext(key.title, side = 3, cex = 1.5, line = 1.5, font = 1),
    add.title = plot.params$add.title,
    plot.title = mtext(text = main.title, side = 3, line = 0, cex = 1.5),
    plot.axes = {
      filled.contour.outline(mcmcpath, longlat, plot.params)
      filled.contour.map(mcmcpath, longlat, plot.params)
      plot.xy
      filled.contour.graph(mcmcpath, longlat, plot.params)
      filled.contour.points(mcmcpath, longlat, plot.params, highlight.samples)
    }
  )
  list(eems.colors = eems.colors, eems.levels = eems.levels)
}

one.prob.contour <- function(mcmcpath, dimns, Props, longlat, plot.params, is.mrates,
                             plot.xy = NULL, highlight.samples = NULL) {
  Props <- (Props + 1) / 2
  Props[Props < 0] <- 0
  Props[Props > 1] <- 1

  if (is.mrates) v <- "m" else v <- "q"
  main.title <- paste0("Posterior probabilities P{log(", v, ") <> 0 | diffs}")
  key.greaterthan0 <- paste0("P{log(", v, ") > 0}")
  key.lessthan0 <- paste0("P{log(", v, ") < 0}")

  # Probabilities are lie in the range [0, 1] but I can experiment with different ways
  # to split the range [0, 1] into bins.
  alpha <- plot.params$prob.levels
  alpha <- alpha[alpha > 0.5 & alpha < 1]
  alpha <- sort(unique(alpha))
  prob.labels <- c("", as.character(rev(alpha)), as.character(alpha), "")
  prob.levels <- c(0, 1 - rev(alpha), alpha, 1)
  prob.colors <- default.eems.colors()
  prob.colors <- colorRampPalette(prob.colors)(length(prob.levels) - 1)

  myfilledContour(
    points_to_raster(Props, dimns, plot.params),
    col = prob.colors, levels = prob.levels, asp = 1,
    add.key = plot.params$add.colbar,
    key.axes = axis(4,
      at = prob.levels, labels = prob.labels,
      tick = FALSE, hadj = 0, line = 1, cex.axis = 1.5
    ),
    key.title = {
      mtext(key.greaterthan0, side = 3, cex = 1.5, line = 1.5, font = 1)
      mtext(key.lessthan0, side = 1, cex = 1.5, line = 1.5, font = 1)
    },
    add.title = plot.params$add.title,
    plot.title = mtext(text = main.title, side = 3, line = 0, cex = 1.5),
    plot.axes = {
      filled.contour.outline(mcmcpath, longlat, plot.params)
      filled.contour.map(mcmcpath, longlat, plot.params)
      # Including `plot.xy` has no effect because it has already been
      # evaluated once to add extra points to the rates plot
      plot.xy
      filled.contour.graph(mcmcpath, longlat, plot.params)
      filled.contour.points(mcmcpath, longlat, plot.params, highlight.samples)
    }
  )
  list(prob.colors = prob.colors, prob.levels = prob.levels)
}

random.eems.contour <- function(mcmcpath, dimns, longlat, plot.params, is.mrates,
                                plot.xy = NULL, highlight.samples = NULL) {
  if (is.mrates) {
    message("Plotting effective migration surface (one posterior draw of m rates)")
    zero_mean <- plot.params$m.zero_mean
    log_scale <- plot.params$m.log_scale
  } else {
    message("Plotting effective diversity surface (one posterior draw of q rates)")
    zero_mean <- plot.params$q.zero_mean
    log_scale <- plot.params$q.log_scale
  }

  voronoi <- read.voronoi(mcmcpath, longlat, is.mrates, log_scale)
  tiles <- voronoi$tiles
  rates <- voronoi$rates
  xseed <- voronoi$xseed
  yseed <- voronoi$yseed

  # Choose one saved posterior draw at random
  niters <- length(tiles)
  riter <- sample(seq_len(niters), 1)
  message(mcmcpath)
  message("Draw #", riter, " (out of ", niters, ")")

  # Jump over stored parameters for draws 1 to (riter - 1)
  skip <- sum(tiles[riter:1][-1])
  now.tiles <- tiles[riter]
  now.rates <- rates[(skip + 1):(skip + now.tiles)]
  now.xseed <- xseed[(skip + 1):(skip + now.tiles)]
  now.yseed <- yseed[(skip + 1):(skip + now.tiles)]

  rslt <- transform_rates(dimns, now.tiles, now.rates, now.xseed, now.yseed, zero_mean)
  Zvals <- rslt$Zvals
  # Actually plot the colored contour plot
  one.eems.contour(mcmcpath[1], dimns, Zvals, longlat, plot.params, is.mrates,
    plot.xy = plot.xy, highlight.samples = highlight.samples
  )
}

average.eems.contours <- function(mcmcpath, dimns, longlat, plot.params, is.mrates,
                                  plot.xy = NULL, highlight.samples = NULL) {
  if (is.mrates) {
    message("Plotting effective migration surface (posterior mean of m rates)")
    zero_mean <- plot.params$m.zero_mean
    log_scale <- plot.params$m.log_scale
  } else {
    message("Plotting effective diversity surface (posterior mean of q rates)")
    zero_mean <- plot.params$q.zero_mean
    log_scale <- plot.params$q.log_scale
  }
  Zmean <- matrix(0, dimns$nxmrks, dimns$nymrks)
  PrGT0 <- matrix(0, dimns$nxmrks, dimns$nymrks)
  PrLT0 <- matrix(0, dimns$nxmrks, dimns$nymrks)
  coord_extra <- nrow(dimns$coord)
  Zmean_extra <- numeric(coord_extra)
  niters <- 0
  # Loop over each output directory in mcmcpath to average the colored contour plots
  for (path in mcmcpath) {
    message(path)
    stopifnot(all(file.exists(file.path(path, c(
      "mcmcmtiles.txt",
      "mcmcmrates.txt",
      "mcmcxcoord.txt",
      "mcmcycoord.txt",
      "mcmcqtiles.txt",
      "mcmcqrates.txt",
      "mcmcwcoord.txt",
      "mcmczcoord.txt"
    )))))
    voronoi <- read.voronoi(path, longlat, is.mrates, log_scale)
    tiles <- voronoi$tiles
    rates <- voronoi$rates
    xseed <- voronoi$xseed
    yseed <- voronoi$yseed
    rslt <- transform_rates(dimns, tiles, rates, xseed, yseed, zero_mean)
    Zmean <- Zmean + rslt$Zvals
    Zmean_extra <- Zmean_extra + rslt$Zvals_extra
    niters <- niters + rslt$niters
    if (zero_mean) {
      PrGT0 <- PrGT0 + rslt$PrGT0
      PrLT0 <- PrLT0 + rslt$PrLT0
    }
  }
  Zmean <- Zmean / niters
  PrGT0 <- PrGT0 / niters
  PrLT0 <- PrLT0 / niters
  Zmean_extra <- Zmean_extra / niters

  # Actually plot the colored contour plot
  # Pass one mcmcpath (shouldn't matter which one) in case adding extra information
  # (demes, edges, etc.) on top of the contour plot.
  rates.raster <- one.eems.contour(
    mcmcpath[1], dimns, Zmean, longlat, plot.params, is.mrates,
    plot.xy = plot.xy, highlight.samples = highlight.samples
  )
  xyz.values <- cbind(dimns$coord, Zmean_extra)
  rates.raster$xyz.values <- xyz.values

  if (zero_mean) {
    raster.prob <- one.prob.contour(
      mcmcpath[1], dimns, PrGT0 - PrLT0, longlat, plot.params, is.mrates,
      plot.xy = plot.xy, highlight.samples = highlight.samples
    )
  }
  rates.raster
}

JtDJ2BandW <- function(JtDJ, sizes = NULL) {
  # There should be NAs on the main diagonal of JtDobsJ -- these elements correspond to demes with
  # a single observed sample. And since there is a single sample taken, the average dissimilarity
  # between two distinct individuals from such demes cannot be observed.
  # Instead, I will use the average within dissimilarity, W, across the demes with multiple samples.
  # This is make a difference only if remove.singletons = FALSE, which is not the default option.
  JtDJ <- as.matrix(JtDJ)
  if (!is.null(sizes)) {
    diag(JtDJ)[sizes < 2] <- mean(diag(JtDJ)[sizes >= 2])
  }
  n <- nrow(JtDJ)
  W <- diag(JtDJ)
  S <- matrix(W, n, n)
  B <- JtDJ - (S + t(S)) / 2
  B <- B[upper.tri(B, diag = FALSE)]
  list(W = W, B = B)
}

geo.distm <- function(coord, longlat, plot.params) {
  if (!longlat) {
    coord <- coord[, c(2, 1)]
  }
  if (!is.null(plot.params$proj.in)) {
    # If the locations are projected, convert them to longitude/latitude pairs
    coord <- sp::SpatialPoints(coord, proj4string = CRS(plot.params$proj.in))
    coord <- sp::spTransform(coord, CRSobj = CRS("+proj=longlat +datum=WGS84"))
    coord <- sp::coordinates(coord)
  }
  Dist <- sp::spDists(coord, longlat = TRUE)
  Dist <- Dist[upper.tri(Dist, diag = FALSE)]
  Dist
}

dist.scatterplot <- function(mcmcpath, longlat, plot.params,
                             remove.singletons = TRUE, add.abline = FALSE,
                             add.r.squared = FALSE, highlight = NULL) {
  message("Plotting average dissimilarities within and between demes")
  for (path in mcmcpath) {
    stopifnot(all(file.exists(file.path(path, c(
      "rdistJtDhatJ.txt",
      "rdistJtDobsJ.txt",
      "rdistoDemes.txt"
    )))))
  }
  nchains <- length(mcmcpath)
  # List of observed demes, with number of samples taken collected
  # Each row specifies: x coordinate, y coordinate, n samples
  oDemes <- scan(file.path(mcmcpath[1], "rdistoDemes.txt"), quiet = TRUE)
  oDemes <- matrix(oDemes, ncol = 3, byrow = TRUE)
  Sizes <- oDemes[, 3]
  nPops <- nrow(oDemes)
  Demes <- seq(nPops)
  if (nPops < 2) {
    message(
      "All individuals sampled from the same deme. ",
      "Check that individual and habitat coordinates are given ",
      "in consistent order (either latitude/longitude or ",
      "longitude/latitude) in the *.outer and *.coord files."
    )
    return(NULL)
  }
  JtDobsJ <- matrix(0, nPops, nPops)
  JtDhatJ <- matrix(0, nPops, nPops)
  for (path in mcmcpath) {
    message(path)
    oDemes1 <- scan(file.path(path, "rdistoDemes.txt"), quiet = TRUE)
    oDemes1 <- matrix(oDemes1, ncol = 3, byrow = TRUE)
    if (sum(dim(oDemes) != dim(oDemes1)) || sum(oDemes != oDemes1)) {
      plot.params$add.demes <- TRUE
      plot.params$add.grid <- TRUE
      plot.params$add.title <- TRUE
      plot.params$add.colbar <- FALSE
      null.eems.contour(mcmcpath[1], longlat, plot.params)
      mtext(
        side = 3, line = 2, cex = 1.3, text =
          "EEMS results for at least two different population grids"
      )
      null.eems.contour(path, longlat, plot.params)
      mtext(
        side = 3, line = 2, cex = 1.3, text =
          "EEMS results for at least two different population grids"
      )
      message("EEMS results for at least two different population grids")
      return(NULL)
    }
    JtDobsJ <- JtDobsJ +
      as.matrix(read.table(file.path(path, "rdistJtDobsJ.txt"), header = FALSE))
    JtDhatJ <- JtDhatJ +
      as.matrix(read.table(file.path(path, "rdistJtDhatJ.txt"), header = FALSE))
  }
  JtDobsJ <- JtDobsJ / nchains
  JtDhatJ <- JtDhatJ / nchains
  colnames(JtDobsJ) <- Demes
  rownames(JtDobsJ) <- Demes
  colnames(JtDhatJ) <- Demes
  rownames(JtDhatJ) <- Demes
  Deme1 <- matrix(Demes, nrow = nPops, ncol = nPops)
  Deme2 <- t(Deme1)
  tempi <- matrix(Sizes, nPops, nPops)
  Small <- pmin(tempi, t(tempi))
  Small <- Small[upper.tri(Small, diag = FALSE)]
  alpha <- Deme1[upper.tri(Deme1, diag = FALSE)]
  beta <- Deme2[upper.tri(Deme2, diag = FALSE)]
  # Under pure isolation by distance, we expect the genetic dissimilarities
  # between demes increase with the geographic distance separating them
  Dist <- geo.distm(oDemes[, 1:2], longlat, plot.params)
  if (sum(Sizes > 1) < 2) {
    # Sizes(alpha) > 1 means that there are at least two individuals assigned to deme alpha
    JtDJ.component <- data.frame(
      alpha.x = oDemes[, 1][alpha],
      alpha.y = oDemes[, 2][alpha],
      beta.x = oDemes[, 1][beta],
      beta.y = oDemes[, 2][beta],
      size = Small,
      col = sub.scattercols(Small),
      fitted = JtDhatJ[upper.tri(JtDhatJ, diag = FALSE)],
      obsrvd = JtDobsJ[upper.tri(JtDobsJ, diag = FALSE)],
      stringsAsFactors = FALSE
    )
    sub.scatterplot("JtDJ", JtDJ.component,
      remove.singletons = FALSE,
      add.abline, add.r.squared
    )
    message(
      "There is one or zero demes with multiple observed individuals; ",
      "plotting dissimilarities between singletons instead."
    )
    return(NULL)
  }
  out1 <- JtDJ2BandW(JtDobsJ, Sizes)
  Wobs <- out1$W
  Bobs <- out1$B
  out2 <- JtDJ2BandW(JtDhatJ)
  What <- out2$W
  Bhat <- out2$B
  B.component <- data.frame(
    alpha.x = oDemes[, 1][alpha],
    alpha.y = oDemes[, 2][alpha],
    beta.x = oDemes[, 1][beta],
    beta.y = oDemes[, 2][beta],
    fitted = Bhat,
    obsrvd = Bobs,
    size = Small,
    col = sub.scattercols(Small),
    stringsAsFactors = FALSE
  )
  W.component <- data.frame(
    alpha.x = oDemes[, 1][Demes],
    alpha.y = oDemes[, 2][Demes],
    fitted = What,
    obsrvd = Wobs,
    size = Sizes,
    col = sub.scattercols(Sizes),
    stringsAsFactors = FALSE
  )
  G.component <- data.frame(
    alpha.x = oDemes[, 1][alpha],
    alpha.y = oDemes[, 2][alpha],
    beta.x = oDemes[, 1][beta],
    beta.y = oDemes[, 2][beta],
    fitted = Dist,
    obsrvd = Bobs,
    size = Small,
    col = sub.scattercols(Small),
    stringsAsFactors = FALSE
  )
  # Code to highlight some demes in the scatterplots of observed vs fitted distances
  # This can help to identify demes that are of special interest among all the other points
  # Since `highlights` is not fully working yet and to avoid notes about "no visible
  # binding for global variable XXX", I have commented the code out.
  sub.scatterplot("Between", B.component, remove.singletons, add.abline, add.r.squared)
  sub.scatterplot("Within", W.component, remove.singletons, add.abline, add.r.squared)
  sub.scatterplot("GeoDist", G.component, remove.singletons, add.abline = FALSE, add.r.squared)
  list(B.component = B.component, W.component = W.component, G.component = G.component)
}

#' Shift spatial object longitudinally
#'
#' The function `sp::recenter` shifts coordinates for a Pacific view by shifting longitudes
#' from `[-180, 180]` to `[0, 360]`. I modified `sp::recenter` to shift longitudes by any degree.
#'
#' @param obj A SpatialPolygons or a SpatialPolygonsDataFrame object.
#' @param degree Number of degrees (positive or negative) to shift.
#' @returns A shifted SpatialPolygons or a SpatialPolygonsDataFrame object.
#' @examples
#' # The input is an EEMS run with the default regular triangular grid without holes
#' extdata <- system.file("extdata", package = "reems")
#' eems_run_with_default_habitat <- file.path(extdata, "EEMS-barrier")
#' # Load the population habitat
#' outer <- read.table(file.path(eems_run_with_default_habitat, "outer.txt"))
#' # Each hole is a ring (simple closed polygon) and the holes don't overlap
#' hole1 <- data.frame(V1 = c(2., 5., 5., 2., 2.), V2 = c(2., 2., 5., 5., 2.))
#' hole2 <- data.frame(V1 = c(6.5, 10., 8., 6.5), V2 = c(2.5, 5., 5., 2.5))
#'
#' # Create the new habitat with holes and explicitly setting its projection as long/lat. 
#' new_habitat_with_holes <- create_habitat_with_holes(outer, list(hole1, hole2))
#' sp::proj4string(new_habitat_with_holes) <- sp::CRS("+proj=longlat +datum=WGS84")
#' recenter_by(new_habitat_with_holes, 60)
#' @noRd
setGeneric("recenter_by", function(obj, degree) {
  standardGeneric("recenter_by")
})

recenter_by.SpatialPolygons <- function(obj, degree) {
  proj <- is.projected(obj)
  if (is.na(proj)) stop("unknown coordinate reference system")
  if (proj) stop("cannot shift projected coordinate reference system")
  projargs <- methods::slot(obj, "proj4string")
  pls <- methods::slot(obj, "polygons")
  Srl <- lapply(pls, recenter_by.Polygons, degree)
  res <- SpatialPolygons(Srl, proj4string = projargs)
  res
}

recenter_by.Polygons <- function(obj, degree) {
  ID <- methods::slot(obj, "ID")
  rings <- methods::slot(obj, "Polygons")
  srl <- lapply(rings, recenter_by.Polygon, degree)
  res <- Polygons(srl, ID = ID)
  res
}


recenter_by.Polygon <- function(obj, degree) {
  crds <- methods::slot(obj, "coords")
  hole <- methods::slot(obj, "hole")
  crds[, 1] <- crds[, 1] + degree
  res <- Polygon(crds, hole)
  res
}

save.graphics <- function(plotpath, plot.params, def.out = NULL, ...) {
    if(is.null(plot.params$out.png)) {
        out <- def.out
    } else {
        out <- plot.params$out.png
    }
    plotpath <- path.expand(plotpath)
  if (out) {
    png(paste0(plotpath, "%02d.png"),
      res = plot.params$res, units = "in",
      height = plot.params$height,
      width = plot.params$width, ...
    )
  } else {
    pdf(paste0(plotpath, "%02d.pdf"),
      height = plot.params$height,
      width = plot.params$width,
      onefile = FALSE, ...
    )
  }
}

plot_with_graphics_device <- function(filename, save.params, expr, def.out = NULL, par.args = list(), ...) {
  save.graphics(plotpath = filename, plot.params = save.params, def.out = def.out, ...)
  oldpar <- graphics::par(no.readonly = TRUE)
  on.exit({
    graphics::par(oldpar)
    grDevices::dev.off()
  }, add = TRUE)

  do.call(graphics::par, par.args)
  force(expr)
}

check.plot.params <- function(params) {
  ## Is there a way to check if a string is a valid PROJ.4 string?
  if (is.character(params$proj.in)) {
    params$proj.in <- params$proj.in[1]
  } else {
    params$proj.in <- NULL
  }
  if (is.character(params$proj.out)) {
    params$proj.out <- params$proj.out[1]
  } else {
    params$proj.out <- params$proj.in
  }
  message(
    "Input projection: ", params$proj.in, "\n",
    "Output projection: ", params$proj.out, "\n\n\n"
  )
  if (is.null(params$proj.in)) {
    if (!is.null(params$proj.out)) {
      stop("Specify the input projection, projection.in, as a PROJ.4 string")
    }
  }

  if (is.logical(params$add.map)) {
    params$add.map <- params$add.map[1]
  } else {
    params$add.map <- FALSE
  }
  if (is.color(params$col.map)) {
    params$col.map <- params$col.map[1]
  } else {
    params$col.map <- "#AAAAAA"
  }
  if (is.numeric(params$lwd.map)) {
    params$lwd.map <- params$lwd.map[1]
  } else {
    params$lwd.map <- 2
  }

  if (is.logical(params$add.grid)) {
    params$add.grid <- params$add.grid[1]
  } else {
    params$add.grid <- FALSE
  }
  if (is.color(params$col.grid)) {
    params$col.grid <- params$col.grid[1]
  } else {
    params$col.grid <- "#BBBBBB"
  }
  if (is.numeric(params$lwd.grid)) {
    params$lwd.grid <- params$lwd.grid[1]
  } else {
    params$lwd.grid <- 1
  }

  if (is.logical(params$add.outline)) {
    params$add.outline <- params$add.outline[1]
  } else {
    params$add.outline <- FALSE
  }
  if (is.color(params$col.outline)) {
    params$col.outline <- params$col.outline[1]
  } else {
    params$col.outline <- "#EEEEEE"
  }
  if (is.numeric(params$lwd.outline)) {
    params$lwd.outline <- params$lwd.outline[1]
  } else {
    params$lwd.outline <- 2
  }

  if (is.logical(params$add.demes)) {
    params$add.demes <- params$add.demes[1]
  } else {
    params$add.demes <- FALSE
  }
  if (is.logical(params$all.demes)) {
    params$all.demes <- params$all.demes[1]
  } else {
    params$all.demes <- FALSE
  }
  if (is.color(params$col.demes)) {
    params$col.demes <- params$col.demes[1]
  } else {
    params$col.demes <- "#000000"
  }
  if (is.numeric(params$pch.demes)) {
    params$pch.demes <- params$pch.demes[1]
  } else {
    params$pch.demes <- 19
  }
  if (is.numeric(params$min.cex.demes)) {
    params$min.cex.demes <- params$min.cex.demes[1]
  } else {
    params$min.cex.demes <- 1
  }
  if (is.numeric(params$max.cex.demes)) {
    params$max.cex.demes <- params$max.cex.demes[1]
  } else {
    params$max.cex.demes <- 3
  }
  if (params$max.cex.demes < params$min.cex.demes) {
    params$max.cex.demes <- params$min.cex.demes
  }

  if (is.logical(params$add.seeds)) {
    params$add.seeds <- params$add.seeds[1]
  } else {
    params$add.seeds <- FALSE
  }
  if (is.color(params$col.seeds)) {
    params$col.seeds <- params$col.seeds[1]
  } else {
    params$col.seeds <- "#8AE234"
  }
  if (is.numeric(params$pch.seeds)) {
    params$pch.seeds <- params$pch.seeds[1]
  } else {
    params$pch.seeds <- 19
  }
  if (is.numeric(params$cex.seeds)) {
    params$cex.seeds <- params$cex.seeds[1]
  } else {
    params$cex.seeds <- 1
  }

  if (params$add.map) {
    load.required.package(package = "rworldmap", required.by = "add.map")
    load.required.package(package = "rworldxtra", required.by = "add.map")
    if (is.null(params$proj.in)) {
      stop(paste0(
        "To add a geographical map, specify the input and output projections.\n",
        "For example, if the coordinates are longitude and latitude, \n",
        "you can use the PROJ.4 string '+proj=longlat +datum=WGS84'\n\n\n"
      ))
    }
  }

  if (is.numeric(params$m.colscale)) {
    params$m.colscale <- set.colscale(params$m.colscale)
  } else {
    params$m.colscale <- c(-2.5, 2.5)
  }
  if (is.numeric(params$q.colscale)) {
    params$q.colscale <- set.colscale(params$q.colscale)
  } else {
    params$q.colscale <- c(-0.1, 0.1)
  }
  if (length(params$eems.colors) < 2 || !all(is.color(params$eems.colors))) {
    params$eems.colors <- default.eems.colors()
  }

  if (is.logical(params$add.title)) {
    params$add.title <- params$add.title[1]
  } else {
    params$add.title <- TRUE
  }
  if (is.logical(params$add.colbar)) {
    params$add.colbar <- params$add.colbar[1]
  } else {
    params$add.colbar <- TRUE
  }

  ## Additional options: By default, both the effectove migration rates and
  ## the effective diversity rates are log10-transformed and mean-zero standardized
  if (is.logical(params$m.zero_mean)) {
    params$m.zero_mean <- params$m.zero_mean[1]
  } else {
    params$m.zero_mean <- TRUE
  }
  if (is.logical(params$q.zero_mean)) {
    params$q.zero_mean <- params$q.zero_mean[1]
  } else {
    params$q.zero_mean <- TRUE
  }
  if (is.logical(params$m.log_scale)) {
    params$m.log_scale <- params$m.log_scale[1]
  } else {
    params$m.log_scale <- TRUE
  }
  if (is.logical(params$q.log_scale)) {
    params$q.log_scale <- params$q.log_scale[1]
  } else {
    params$q.log_scale <- TRUE
  }

  params$prob.levels <- params$prob.levels[params$prob.levels >= 0]
  params$prob.levels <- params$prob.levels[params$prob.levels <= 1]
  params$prob.levels <- sort(unique(c(0, params$prob.levels, 1)))

  params
}

sub.axes.labels <- function() {
  JtDJ <- list(
    xlab = expression(paste("Fitted dissimilarity between individuals  ", Delta[i * j])),
    ylab = expression(paste("Observed dissimilarity between individuals  ", D[i * j])),
    mainTRUE = "There should be at least two observed demes to plot pairwise dissimilarities"
  )
  Between <- list(
    xlab = expression(paste(
      "Fitted dissimilarity between demes:   ",
      Delta[alpha * beta], " - (", Delta[alpha * alpha], "+", Delta[beta * beta], ") / 2"
    )),
    ylab = expression(paste(
      "Observed dissimilarity between demes:   ",
      D[alpha * beta], " - (", D[alpha * alpha], "+", D[beta * beta], ") / 2"
    )),
    mainTRUE = expression(paste(
      "Dissimilarities between pairs of sampled demes (",
      alpha, ", ", beta, ")"
    )),
    subTRUE = "Singleton demes, if any, are excluded from this plot (but not from EEMS)",
    mainFALSE = expression(paste(
      "Dissimilarities between pairs of sampled demes (",
      alpha, ", ", beta, ")"
    )),
    subFALSE = expression(paste(
      "Gray means that a single individual is sampled from either ",
      alpha, " or ", beta
    ))
  )
  Within <- list(
    xlab = expression(paste("Fitted dissimilarity within demes:   ", Delta[alpha * alpha])),
    ylab = expression(paste("Observed dissimilarity within demes:   ", D[alpha * alpha])),
    mainTRUE = expression(paste("Dissimilarities within sampled demes ", alpha)),
    subTRUE = "Singleton demes, if any, are excluded from this plot (but not from EEMS)",
    mainFALSE = expression(paste("Dissimilarities within sampled demes ", alpha))
  )
  GeoDist <- Between
  GeoDist$xlab <- "Great circle distance between demes (km)"
  list(JtDJ = JtDJ, Between = Between, Within = Within, GeoDist = GeoDist)
}

# dist.type takes the values: "JtDJ", "Between", "Within", "GeoDist"
dist.axes.labels <- function(dist.type, remove.singletons = TRUE, subtitle = NULL) {
  labels <- sub.axes.labels()[[dist.type]]
  title(xlab = labels$xlab)
  title(ylab = labels$ylab)
  if (remove.singletons) {
    mtext(side = 3, line = 2, cex = 1.3, text = labels$mainTRUE)
    if (is.null(subtitle)) {
      mtext(side = 3, line = 0.5, cex = 1, text = labels$subTRUE)
    } else {
      mtext(side = 3, line = 0.5, cex = 1, text = subtitle)
    }
  } else {
    mtext(side = 3, line = 2, cex = 1.3, text = labels$mainFALSE)
    if (is.null(subtitle)) {
      mtext(side = 3, line = 0.5, cex = 1, text = labels$subFALSE)
    } else {
      mtext(side = 3, line = 0.5, cex = 1, text = subtitle)
    }
  }
}

sub.scattercols <- function(sizes) {
  c("black", "gray60")[1 + (sizes < 2)]
}

sub.scatterplot <- function(dist.type, dist.data, remove.singletons, add.abline, add.r.squared,
                            subtitle = NULL, add = FALSE) {
  if (remove.singletons) {
    dist.data <- dist.data[dist.data$size > 1, ]
  }
  if (is.null(dist.data$size)) dist.data$size <- 2
  if (is.null(dist.data$pch)) dist.data$pch <- 1
  if (is.null(dist.data$cex)) dist.data$cex <- 1
  if (is.null(dist.data$col)) dist.data$col <- 1

  group <- dist.data$col
  # It turns out sort.list sorts alphabetically by group name
  # The following works to sort by number of occurrences:
  ord <- sort.list(table(group)[group], decreasing = TRUE)
  dist.data <- dist.data[ord, ]

  if (!add) {
    plot(dist.data$fitted, dist.data$obsrvd, type = "n", xlab = "", ylab = "")
    dist.axes.labels(dist.type, remove.singletons, subtitle)
    if (add.abline) abline(a = 0, b = 1, col = "red", lwd = 2)
    if (add.r.squared) {
      # Fit a linear model for the observed dissimilarities as a function of the fitted ones
      r.squared <- summary(lm(dist.data$obsrvd ~ dist.data$fitted))$r.squared
      r.squared <- round(r.squared, digits = 3)
      legend("topleft", legend = substitute(
        paste(R^2, " = ", val),
        list(val = r.squared)
      ), bty = "n")
    }
  }
  points(dist.data$fitted,
    dist.data$obsrvd,
    col = dist.data$col,
    pch = dist.data$pch,
    cex = dist.data$cex
  )
}

# The distance metric used by EEMS
# Euclidean distance by default; great circle (haversine) distance as an alternative
which.dist.metric <- function(mcmcpath) {
  dist.metric <- "euclidean"
  Lines <- readLines(file.path(mcmcpath, "eemsrun.txt"))
  for (i in seq(Lines)) {
    s <- gsub("\\s", "", Lines[i]) # Remove any empty space
    x <- strsplit(s, "distance = ")[[1]] # Is there a line 'distance = xxx'?
    if (length(x) == 2) dist.metric <- tolower(x[2]) # What is the distance metric?
  }
  if ((dist.metric == "euclidean") || (dist.metric == "greatcirc")) {
    message(
      "Using '", dist.metric,
      "' distance to assign interpolation points to Voronoi tiles.\n\n\n"
    )
  } else {
    stop("Specify either 'euclidean' or 'greatcirc' distance metric in eemsrun.txt.")
  }
  dist.metric
}

read.dimns <- function(path, longlat, nxmrks = NULL, nymrks = NULL, coord = NULL) {
  eems.output <- NULL
  datapath.outer <- paste0(path, ".outer")
  mcmcpath.outer <- file.path(path, "outer.txt")
  if (file.exists(mcmcpath.outer)) {
    eems.output <- TRUE
  } else if (file.exists(datapath.outer)) {
    eems.output <- FALSE
  }
  if (is.null(eems.output)) {
    stop(paste0(path, " is neither a datapath nor a mcmcpath."))
  }
  if (eems.output) {
    outer <- scan(mcmcpath.outer, what = numeric(), quiet = TRUE)
  } else {
    outer <- scan(datapath.outer, what = numeric(), quiet = TRUE)
  }
  outer <- matrix(outer, ncol = 2, byrow = TRUE)
  if (is.null(coord)) {
    coord <- matrix(0, ncol = 2, nrow = 0)
  } else {
    coord <- as.matrix(coord, ncol = 2, byrow = TRUE)
  }
  # "Close" the outline if the first row is not the same as the last row
  if (sum(head(outer, 1) != tail(outer, 1))) {
    outer <- rbind(outer, head(outer, 1))
  }
  if (!longlat) {
    outer <- outer[, c(2, 1)]
    coord <- coord[, c(2, 1)]
  }
  xlim <- range(outer[, 1])
  ylim <- range(outer[, 2])
  aspect <- abs((diff(ylim) / diff(xlim)) / cos(mean(ylim) * pi / 180))
  # Choose the number of interpolation in each direction
  if (is.null(nxmrks) && is.null(nymrks)) {
    if (aspect > 1) {
      nxmrks <- 100
      nymrks <- round(nxmrks * aspect)
    } else {
      nymrks <- 100
      nxmrks <- round(nymrks / aspect)
    }
  }
  # The interpolation points are equally spaced
  xmrks <- seq(from = xlim[1], to = xlim[2], length = nxmrks)
  ymrks <- seq(from = ylim[1], to = ylim[2], length = nymrks)
  marks <- cbind(rep(xmrks, times = nymrks), rep(ymrks, each = nxmrks))
  if (eems.output) {
    dist.metric <- which.dist.metric(path)
  } else {
    dist.metric <- "euclidean"
  }
  # If no additional coordinates at which to estimate m and q rates
  # are specified, report the m and q estimates at the raster marks
  if (!nrow(coord)) {
    coord <- marks
  }
  list(
    nxmrks = nxmrks, xmrks = xmrks, xlim = xlim, xspan = diff(xlim),
    nymrks = nymrks, ymrks = ymrks, ylim = ylim, yspan = diff(ylim),
    marks = marks, nmrks = c(nxmrks, nymrks), aspect = aspect,
    outer = outer, coord = coord, dist.metric = dist.metric
  )
}

read.edges <- function(mcmcpath) {
  edges <- read.table(file.path(mcmcpath, "edges.txt"), colClasses = numeric())
  edges <- as.matrix(edges)
  # Previously EEMS output the edges with one vertex per line and
  # the six neighbors of each vertex listed in order
  # Currently EEMS outputs the edges with one edge per line (as a
  # pair of vertices) which is more general
  if (ncol(edges) == 6) {
    # Convert the old format to the new format
    edges0 <- edges
    edges <- matrix(0, nrow = sum(edges0 > 0), ncol = 2)
    nv <- nrow(edges0)
    nn <- ncol(edges0)
    nodes <- 1:nv
    e <- 0
    for (a in 1:nv) {
      for (i in 1:nn) {
        b <- edges0[a, i]
        if (b %in% nodes) {
          e <- e + 1
          edges[e, 1] <- a
          edges[e, 2] <- b
        }
      }
    }
  }
  edges
}

read.graph <- function(path, longlat) {
  eems.output <- NULL
  if (
    file.exists(file.path(path, "demes.txt")) &&
      file.exists(file.path(path, "ipmap.txt")) &&
      file.exists(file.path(path, "outer.txt"))
  ) {
    eems.output <- TRUE
  } else if (
    file.exists(paste0(path, ".coord")) &&
      file.exists(paste0(path, ".diffs")) &&
      file.exists(paste0(path, ".outer"))
  ) {
    eems.output <- FALSE
  }
  if (is.null(eems.output)) {
    stop(paste0(path, " is neither a datapath nor a mcmcpath."))
  }
  if (eems.output) {
    # Read the assigned sample coordinates
    ipmap <- scan(file.path(path, "ipmap.txt"), what = numeric(), quiet = TRUE)
    demes <- scan(file.path(path, "demes.txt"), what = numeric(), quiet = TRUE)
    outer <- scan(file.path(path, "outer.txt"), what = numeric(), quiet = TRUE)
    demes <- matrix(demes, ncol = 2, byrow = TRUE)
    outer <- matrix(outer, ncol = 2, byrow = TRUE)
    edges <- read.edges(path)
  } else {
    # Read the original sample coordinates
    coord <- scan(paste0(path, ".coord"), what = numeric(), quiet = TRUE)
    outer <- scan(paste0(path, ".outer"), what = numeric(), quiet = TRUE)
    coord <- matrix(coord, ncol = 2, byrow = TRUE)
    outer <- matrix(outer, ncol = 2, byrow = TRUE)
    edges <- NULL
    # The two sampling coordinates are combined to define a "deme",
    # with the maximum possible precision
    ipmap <- factor(paste(coord[, 1], coord[, 2], sep = "x"))
    demes <- levels(ipmap)
    index <- match(demes, ipmap)
    o <- length(index)
    demes <- coord[index, ]
    ipmap <- (1:o)[ipmap]
    # "Close" the outline if the first row is not the same as the last row
    if (sum(head(outer, 1) != tail(outer, 1))) {
      outer <- rbind(outer, head(outer, 1))
    }
  }
  if (!longlat) {
    demes <- demes[, c(2, 1)]
    outer <- outer[, c(2, 1)]
  }
  sizes <- table(ipmap)
  alpha <- as.numeric(names(sizes))
  sizes <- as.numeric(sizes)
  list(ipmap = ipmap, demes = demes, edges = edges, alpha = alpha, sizes = sizes, outer = outer)
}

read.voronoi <- function(mcmcpath, longlat, is.mrates, log_scale) {
  if (is.mrates) {
    rates <- scan(file.path(mcmcpath, "mcmcmrates.txt"),
      what = numeric(), quiet = TRUE
    )
    tiles <- scan(file.path(mcmcpath, "mcmcmtiles.txt"),
      what = numeric(), quiet = TRUE
    )
    xseed <- scan(file.path(mcmcpath, "mcmcxcoord.txt"),
      what = numeric(), quiet = TRUE
    )
    yseed <- scan(file.path(mcmcpath, "mcmcycoord.txt"),
      what = numeric(), quiet = TRUE
    )
  } else {
    rates <- scan(file.path(mcmcpath, "mcmcqrates.txt"),
      what = numeric(), quiet = TRUE
    )
    tiles <- scan(file.path(mcmcpath, "mcmcqtiles.txt"),
      what = numeric(), quiet = TRUE
    )
    xseed <- scan(file.path(mcmcpath, "mcmcwcoord.txt"),
      what = numeric(), quiet = TRUE
    )
    yseed <- scan(file.path(mcmcpath, "mcmczcoord.txt"),
      what = numeric(), quiet = TRUE
    )
  }
  if (!longlat) {
    tempi <- xseed
    xseed <- yseed
    yseed <- tempi
  }
  if (log_scale) {
    rates <- log10(rates)
  }
  list(rates = rates, tiles = tiles, xseed = xseed, yseed = yseed)
}

---
title: "reems"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{reems}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

```{r setup}
library(reems)
```
This vignette will walk you through the procedure of estimating effective migration surfaces from single-nucleotide polymorphism data (SNPs). It is designed to fit within the spatial population genetics ecosystem and to be compatible in its input format with other `R` packages such as [`conStruct`](https://CRAN.R-project.org/package=conStruct).

The best introduction to the EEMS method itself is the original article by Petkova *et al.* <doi:10.1038/ng.3464>, a preprint of which is also available on bioRxiv (`https://www.biorxiv.org/content/10.1101/011809v2.full`). 
Both the preprint version of the original article and the documentation of the original command-line utility are also available on [their GitHub repository](https://github.com/dipetkov/eems/tree/master/Documentation) and contain  many high-resolution examples of what the EEMS outplot plots look like. 

At the beginning of this process, we assume you have an *allele frequency matrix* `freqs` and a *coordinate matrix* `coords` in your workspace. This package includes example objects `ex.freqs` and `ex.coords`. 

# Estimating Effective Migration Surfaces 

## Standard usage

The simplest way to run an EEMS analysis is by using the `eems()` function of this package.
It can be invoked with only minimal arguments: 
```{r}
eems(
  freqs = freqs,
  coords = coords,
  mcmcpath = "/full/path/to/directory/prefix"
)	
```
This runs an EEMS analysis for a set of diploid individuals on a single MCMC chain for 2000000 iterations, and with a default area outline derived from the coordinates themselves by adding a buffer around them. 
It also assumes a default number of demes 6 times the number of individuals. 
The results will be stored in an output directory `full/path/to/file/prefix-chain1`.
The return value of the function is the output directory, so it is easy to retrieve.  

Those defaults are designed to ensure an adequate resolution of the analysis, but they may not be suitable for your exact application context. In that case, you can set those parameters manually by specifying additional arguments. 
The most useful ones are illustrated below, where the MCMC chain has been shortened by an order of magnitude for a more exploratory analysis on haploid samples. It also shows that the seed can be set manually to ensure reproducibility. 
```{r}
eems(
  seed = 1024,	
  freqs = freqs,
  coords = coords,
  mcmcpath = "/full/path/to/directory/prefix",
  diploid = FALSE,
  numMCMCIter = 100000,
  numBurnIter = 50000,
  numThinIter = 999
)	
```
Usually, it is advisable to run more than one MCMC chain and combine the results. 
`eems()` allows you to do so either sequentially or in parallel. 
This example will run 5 chains in parallel with default settings, although it will not use more workers than there are cores available. Note that when run in parallel, the chains will run silently and console output is redirected to a log file which you can find among the other output files in the output directory.  
```{r}
eems(
  freqs = freqs,
  coords = coords,
  mcmcpath = "full/path/to/directory/prefix",
  nChains = 5,
  parallel = TRUE
)	
```
The output will then be in 5 separate directories, called `/full/path/to/directory/prefix-chain1` to `full/path/to/directory/prefix-chain5`. 

## Specifying a more complex habitat

Sometimes, one wants to use a more complex habitat which cannot be described as everything within a single outline.
In those cases, we can use the function `grid.make()` to create a grid that has both boundary points and holes.
This grid can then be passed to `eems()` using the optional parameters `demes` and `edges`.
The following code loads an example outline and places two holes inside it before running an EEMS analysis with the input.     

```{r}
data_path <- system.file("extdata", package = "reems")
input <- file.path(data_path, "barrier-schemeX-nIndiv300-nSites3000")

outer <- read.table(paste0(input, ".outer"))

hole1 <- data.frame(V1 = c(2., 5., 5., 2., 2.), V2 = c(2., 2., 5., 5., 2.))
hole2 <- data.frame(V1 = c(6.5, 10., 8., 6.5), V2 = c(2.5, 5., 5., 2.5))

# Create the new grid  with holes
grid <- grid.make(outer = outer, holes = list(hole1, hole2), nDemes = 300)

eems(
  freqs = freqs,
  coords = coords,
  mcmcpath = "full/path/to/directory/prefix",
  demes = grid$demes,
  edges = grid$edges,
  nChains = 5,
  parallel = TRUE
)	

```

## Hyperparameters

Various additional parameters can be set with additional arguments to `eems()`, documentation of which can be found in the function documentation  for `eems()`. This includes the various hyperparameters available for tuning. 

```{r}
?eems
```
Regarding those, in the documentation of the `EEMS` command-line utility, available at  <https://github.com/dipetkov/eems/blob/master/Documentation/EEMS-doc.pdf>, Petkova writes the following:

-----

Choosing values for the proposal variances is something of a dark art– the goal is to
choose the parameters so that proposals are accepted about 20% 30% of the time. In practice, it seems to be
sufficient to choose the variances so that proposals are not accepted too rarely (less than 10% of the time) or
too often (more than 40% of the time).

Suppose that we run EEMS (say, with the default values for all proposal distributions) and at the end EEMS
reports the following information about the frequency of accepting proposals of different types.

```
1 (2050/2593) = 79.06% for type "qTileRate"
## change the rate of one tile in the diversity tessellation
2 (1414/2478) = 57.06% for type "qTileMove"
## move the seed of one tile in the diversity tessellation
3 (1047/2494) = 41.98% for type "qBirthDeath"
## add/remove one tile in the diversity tessellation
4 (20625/47530) = 43.39% for type "mTileRate"
## change the rate of one tile in the migration tessellation
5 (15486/50098) = 30.91% for type "mMeanRate"
## change the overall log10 migration rate
6 (22352/47532) = 47.03% for type "mTileMove"
## move the seed of one tile in the migration tessellation
7 (26305/47275) = 55.64% for type "mBirthDeath"
## add/remove one tile in the migration tessellation
```
It seems that `qTileRate` and `qTileMove` are accepted too often. So it might be a good idea to increase
`qEffctProposalS2` and `qSeedsProposalS2`. [If the proposal variance is higher, then bigger changes/moves will
be proposed and hence the updates will be accepted less often.]

Finally, the parameter `qVoronoiPr` specifies how often to propose updating the diversity Voronoi tessellation.
Its default value is `0.05`, which means that about `5%` of the time the proposals are about the diversity tessellation
and about `95%` of the time the proposals are about the migration tessellation.

-----

# Plotting the output of an EEMS analysis

The output of the EEMS analysis performed with `eems()` can be plotted using `eems.plots()`, which produces several figures:

* `mrates01` is the effective migration surface. 
This contour plot visualizes the estimated effective migration rates $m$, on the log10 scale after mean centering.

* `mrates02` shows  posterior probabilities $P(m > 0 | \texttt{diffs})$ and $P(m < 0 | \texttt{diffs})$ for each location in the habitat. 
Since migration rates are visualized on the log10 scale after mean centering, 0 corresponds to the overall mean migration rate. 
This contour plot emphasizes regions with effective migration that is significantly higher/lower than the overall average.

* `qrates01` plots the effective diversity surface. This contour plot visualizes the estimated effective diversity rates $q$, on the log10 scale after mean centering.
* `qrates02` shows the posterior probabilities $P(q > 0 | \texttt{diffs})$ and $P(q < 0 | \texttt{diffs})$. Similar to `mrates02` but applied to the effective diversity rates.
* `rdist01` is a scatter plot of the observed vs the fitted between-deme component of genetic dissimilarity, where one point represents a pair of sampled demes.
* `rdist02` is a scatter plot of the observed vs the fitted between-deme component of genetic dissimilarity, where one point represents a sampled deme.
* `rdist03` is a scatter plot of observed genetic dissimilarities between demes vs observed geographic distances between demes.
* `pilogl01` is the posterior probability trace plot. 

The `mrates` and `qrates` figures visualize (properties of) the effective migration
and diversity rates across the habitat. The other figures can help to check that the MCMC
sampler has converged (the trace plot `pilogl`) and that the EEMS model fits the data
well (the scatter plots of genetic dissimilarities `rdist`).

The function `eems.plots()` will work given the results from a single EEMS run (one
directory in `mcmcpath`) but it is better to run EEMS several times, randomly initializing
the MCMC chain each time.

Hence, `eems.plots()` takes as input a vector of EEMS output directories. 
If the chains have been obtained from a single call of `eems()` with `nChains` greater than 1, or if only a single run is to be plotted, 
one can use the return value of `eems()`, which is precisely this vector of output directories. 
The other required arguments for `eems.plots()` are `plotpath`, which is the full path to a file prefix that will then be completed by the names of the plots given above, and `longlat`, a logical value that is set `TRUE` if the X coordinate is the longitude and the Y coordinate the latitude, and `FALSE` otherwise. 
```{r}
eems_results <- eems(
  freqs = freqs,
  coords = coords,
  mcmcpath = "/full/path/to/directory/prefix",
  nChains = 5,
  parallel = TRUE
)	

eems.plots(
  mcmcpath = eems_results,
  plotpath = "/full/path/to/file/prefix",
  longlat = TRUE
)
```

Various optional parameters add additional plotting functionality, which we will illustrate with a series of examples. 

We can add the original sampling locations on top of the contour plot.
```{r}
eems.plots(
  mcmcpath = eems_results,
  plotpath = "/full/path/to/file/with-sampling",
  longlat = TRUE,
  m.plot.xy = {
    points(coord, col = "purple")
  },
  q.plot.xy = {
    points(coord, col = "purple")
  }
)
```

We can flip the x and y axis, i.e., assume that the x coordinate is the latitude and the y coordinate is the longitude.
```{r}
eems.plots(
  mcmcpath = eems_results,
  plotpath =  "/full/path/to/file/axes-flipped",
  longlat = FALSE
)
```

We can also control the output size and format.
The code below generates all figures as PDF, with the `mrates` and `qrates` plots of height 12 inches and width 10 inches.
```{r}
eems.plots(
  mcmcpath = eems_results,
  plotpath =  "/full/path/to/file/output-PDFs",
  longlat = TRUE,
  plot.height = 12,
  plot.width = 10
)
```

The following code will generate all figures as PNG, with the `mrates` and `qrates` plots of height  9 inches, width 8 inches and resolution 600 dots per inch.
```{r}
eems.plots(
  mcmcpath = eems_results,
  plotpath =  "/full/path/to/file/output-PNGs",
  longlat = TRUE,
  plot.height = 9,
  plot.width = 8,
  res = 600,
  out.png = TRUE
)
```

We can set various colors and shapes for the outline, the grid and the demes.
```{r}
eems.plots(
  mcmcpath = eems_results,
  plotpath =  "/full/path/to/file/demes-and-edges",
  longlat = TRUE,
  add.grid = TRUE,
  col.grid = "gray90",
  lwd.grid = 2,
  add.outline = TRUE,
  col.outline = "blue",
  lwd.outline = 5,
  add.demes = TRUE,
  col.demes = "red",
  pch.demes = 5,
  min.cex.demes = 0.5,
  max.cex.demes = 1.5
)
```

If we have the `RColorBrewer` package, we can also use a divergent Red to Blue color scheme instead of the default DarkOrange to Blue color scheme.
```{r}
eems.plots(
  mcmcpath = eems_results,
  plotpath = "/full/path/to/file/new-eems-colors",
  longlat = TRUE,
  eems.colors = RColorBrewer::Rbrewer.pal(11, "RdBu")
)
```
In a similar vein, we can specify the color scales for the migration and diversity rates.
```{r}
eems.plots(
  mcmcpath = eems_results,
  plotpath = "/full/path/to/file/fix-colscales",
  longlat = TRUE,
  add.outline = TRUE,
  col.outline = "gray",
  m.colscale = c(-3, 3),
  q.colscale = c(-0.3, +0.3)
)
```

Often, especially when dealing with larger spatial scales, it is helpful to produce contour plots that follow a certain cartographic projection, in this case the Mercator projection.
```{r}
projection_none <- "+proj=longlat +datum=WGS84"
projection_mercator <- "+proj=merc +datum=WGS84"

eems.plots(
  mcmcpath = eems_results,
  plotpath = "/full/path/to/file/cartographic-projections",
  longlat = TRUE,
  projection.in = projection_none,
  projection.out = projection_mercator
)
```
Using the additional packages `rworldmap` and `rworldxtra`, we can even add a high-resolution geographic map as a backdrop to our plots. Setting the corresponding option `add.map = TRUE` requires both those packages, and also requires that `projection.in` is specificed. 
```{r}
eems.plots(
  mcmcpath = eems_results,
  plotpath = "/full/path/to/file/geographic-map",
  longlat = TRUE,
  projection.in = projection_none,
  projection.out = projection_mercator,
  add.map = TRUE,
  col.map = "black",
  lwd.map = 5
)
```
We can explicitly add a specific map by passing the shape file.
```{r}
map_world <- rworldmap::getMap()
map_africa <- map_world[which(map_world@data$continent == "Africa"), ]
eems.plots(
  mcmcpath = eems_results,
  plotpath = "/full/path/to/file/shapefile",
  longlat = TRUE,
  m.plot.xy = {
    plot(map_africa, col = NA, add = TRUE)
  },
  q.plot.xy = {
    plot(map_africa, col = NA, add = TRUE)
  }
)
```
Combining both, we can apply the Mercator projection and add the map of Africa. 
We must be careful to apply the same projection to the map as well.
```{r}
map_africa <- sp::spTransform(map_africa,  sp::CRS(projection_mercator))
eems.plots(
  mcmcpath = eems_results,
  plotpath =  "/full/path/to/file/shapefile-projected",
  longlat = TRUE,
  projection.in = projection_none,
  projection.out = projection_mercator,
  m.plot.xy = {
    plot(map_africa, col = NA, add = TRUE)
  },
  q.plot.xy = {
    plot(map_africa, col = NA, add = TRUE)
  }
)
```
Similarly, we can add points, lines, labels, etc. 
Here is how to add a set of colored "labels" on top of the migration/diversity rates and the Africa map. 
Note that we also need to project any points we want to plot. 
```{r}
coords <- matrix(c(
  -10, 10,
  10, 10,
  30, 0,
  40, -10,
  30, -20
), ncol = 2, byrow = TRUE)
colors <- c("red", "green", "blue", "purple", "orange")
labels <- LETTERS[1:5]
coords_merc <- sp::spTransform(
  sp::SpatialPoints(coords, sp::CRS(projection_none)),
  sp::CRS(projection_mercator)
)

coords_merc <- coords_merc@coords

eems.plots(
  mcmcpath = eems_results,
  plotpath = "/full/path/to/file/labels-projected",
  longlat = TRUE,
  projection.in = projection_none,
  projection.out = projection_mercator,
  m.plot.xy = {
    text(coords_merc, col = colors, pch = labels, font = 2)
  },
  q.plot.xy = {
    text(coords_merc, col = colors, pch = labels, font = 2)
  }
)
```

Lastly, we can also use `eems.plots` to compute the migration and diversity rates at specific points. 
They will then be saved in an RData file together with the figures. 
```{r}
xy.coords <- matrix(c(
  13.70, 3.20,
  37.10, -7.20,
  36.10, -4.10,
  34.58, -5.67
), ncol = 2, byrow = TRUE)
eems.plots(
  mcmcpath = eems_results,
  plotpath = "/full/path/to/file/default",
  longlat = TRUE,
  xy.coords = xy.coords
)
load(paste0("/full/path/to/file/default", "-rdist.RData"))
```
The RData file contains `xym.values` and `xyq.values`.
`xym.values` store the migration rate for each coordinate, on the log10 scale and after mean-centering.
Similarly, `xyq.values` stores the diversity rate for each coordinate, on the log10 scale and after mean-centering.

An overview of all the parameters to `eems.plots()` can be found in its function documentation.
```{r}
?eems.plots
```

# Other functions
Besides the two main user-facing functions `eems()` and `eems.plots()`, which should suffice for the more common use cases,  this package also provides some additional functions, which we will just describe very briefly here. 
More information  about them can be found in their respective function documentation. 

* `eems.from.files()` runs an EEMS analysis from input data available as tabular files on disk, and corresponds precisely to the original EEMS utility available at <https://github.com/dipetkov/eems>. 

* `eems.population.grid()` plots the population grid, with all edges in the same color, to visualize the grid before estimating migration and diversity rates. 
This is sometimes useful for troubleshooting issues with grid connectivity. 
    
* `eems.voronoi.samples()` and `eems.posterior.draws()` both take random samples from the posterior distribution of the migration and diversity rates and plot them to visualize the posterior variance (in a slightly different way).

* `grid.make()` constructs a regular triangular grid with circa nDemes vertices within a habitat outline. 
It can also be used to make a more complex habitat with holes. 

* `outer.buffered()` computes the boundary polygon of a habitat encompassing a set of points, while keeping a specific buffer distance from the supplied points. 
This is used internally by `eems()` to create a default habitat, but can also be used directly to make the outer boundary of a more complex habitat. 

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eems.population.grid.R
\name{eems.population.grid}
\alias{eems.population.grid}
\title{A function to plot the constructed population grid and optionally the initial sampling locations}
\usage{
eems.population.grid(
  mcmcpath,
  plotpath,
  longlat,
  plot.width = 10,
  plot.height = 10,
  out.png = FALSE,
  res = 600,
  add.grid = TRUE,
  col.grid = "gray80",
  lwd.grid = 1,
  add.demes = FALSE,
  col.demes = "black",
  pch.demes = 19,
  min.cex.demes = 1,
  max.cex.demes = 3,
  add.outline = TRUE,
  col.outline = "gray90",
  lwd.outline = 2,
  add.coord = FALSE,
  col.coord = "red",
  pch.coord = 3,
  datapath = NULL
)
}
\arguments{
\item{mcmcpath}{A vector of EEMS output directories, for the same dataset. Warning: There is
minimal checking that the given  directories are for the same dataset.}

\item{plotpath}{The full path and the file name for the graphics to be generated.}

\item{longlat}{A logical value indicating whether the coordinates are given as pairs
(longitude, latitude) or (latitude, longitude).}

\item{plot.width}{The width of the graphics region for the two rate contour plots, in inches.
The default value is 10.}

\item{plot.height}{The height of the graphics region, in inches. The default value is 10.}

\item{out.png}{A logical value which, if set,  forces output graphics to be generated as PNGs
(if \code{TRUE}) or PDFs (if \code{FALSE}). Defaults to \code{FALSE}.}

\item{res}{Resolution, in dots per inch; used only for PNG images.
The default is 600.}

\item{add.grid}{A logical value indicating whether to add the population grid or not.}

\item{col.grid}{The color of the population grid. Defaults to \code{gray80}.}

\item{lwd.grid}{The line width of the population grid. Defaults to 1.}

\item{add.demes}{A logical value indicating whether to add the observed demes or not.}

\item{col.demes}{The color of the demes. Defaults to \code{black}.}

\item{pch.demes}{The symbol, specified as an integer, or the character to be used for
plotting the demes. Defaults to 19.}

\item{min.cex.demes}{The minimum size of the deme symbol/character.}

\item{max.cex.demes}{The maximum size of the deme symbol/character. Defaults to 1 and 3,
respectively. If \code{max.cex.demes} > \code{min.cex.demes}, then demes with more samples
also have bigger size: the deme with the fewest samples has size \code{min.cex.demes} and
the deme with the most samples has size \code{max.cex.demes}.}

\item{add.outline}{A logical value indicating whether to add the habitat outline or not.}

\item{col.outline}{The color of the habitat outline. Defaults to \code{white}.}

\item{lwd.outline}{The line width of the habitat outline. Defaults to 2.}

\item{add.coord}{A logical value indicating whether to add the original sampling locations to
the plot or not.}

\item{col.coord}{The color of the sampling locations. Defaults to \code{red}.}

\item{pch.coord}{The symbol, specified as an integer, or the character to be used for plotting
the sampling locations. Defaults to 3.}

\item{datapath}{The full path and the file name of the input dataset (the three files
datapath.coord, datapath.diffs, datapath.outer). Must be specified if \code{add_coord = TRUE}.}
}
\value{
Passes on the return value of \code{\link[=dev.off]{dev.off()}} indicating the graphics device.
}
\description{
Given an EEMS output directory, this function generates one figure to visualize the EEMS
population grid. All edges are shown in the same color to visualize the grid before estimating
migration and diversity rates. This can be helpful if EEMS exits with the error message
"The population grid is not connected".
}
\examples{
# Use the provided example or supply the path to your own EEMS run.
extdata_path <- system.file("extdata", package = "reems")
eems_results <- file.path(extdata_path, "EEMS-example")
# Create a temporary output directory for this example 
outdir <- file.path(tempdir(), "path_out")
dir.create(outdir, showWarnings = FALSE)
name_figures <- file.path(outdir, "EEMS-grid_connected")

eems.population.grid(eems_results,
  name_figures,
  longlat = TRUE,
  add.outline = TRUE,
  col.outline = "purple",
  lwd.outline = 3,
  add.grid = TRUE,
  col.grid = "green",
  lwd.grid = 2,
  out.png = FALSE
)

# It is more interesting to see an example where the grid is unconnected
# due to the unusual shape of the habitat.
eems_results <- file.path(extdata_path, "EEMS-popgrid")

name_figures <- file.path(outdir, "EEMS-grid_not_connected")

eems.population.grid(
  mcmcpath = eems_results,
  plotpath = name_figures,
  longlat = FALSE,
  add.outline = TRUE, col.outline = "purple", lwd.outline = 3,
  add.grid = TRUE, col.grid = "green", lwd.grid = 2
)
# Delete the output file to tidy up.
unlink(outdir, recursive = TRUE, force = TRUE)
}
\seealso{
\code{\link{eems.plots}, \link{eems.voronoi.samples}, \link{eems.posterior.draws}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eems.run.R
\name{eems.from.files}
\alias{eems.from.files}
\title{Run an EEMS analysis}
\usage{
eems.from.files(
  seed = unclass(Sys.time()),
  datapath,
  mcmcpath,
  prevpath = "",
  gridpath = "",
  nDemes,
  nIndiv,
  nSites,
  diploid = TRUE,
  distance = "euclidean",
  numMCMCIter,
  numBurnIter,
  numThinIter,
  mSeedsProposalS2 = 0.01,
  qSeedsProposalS2 = 0.1,
  mEffctProposalS2 = 0.1,
  qEffctProposalS2 = 0.001,
  mrateMuProposalS2 = 0.01,
  qVoronoiPr = 0.25,
  qrateShape = 0.001,
  mrateShape = 0.001,
  sigmaShape = 0.001,
  qrateScale = 1,
  mrateScale = 1,
  sigmaScale = 1,
  negBiProb = 0.67,
  negBiSize = 10
)
}
\arguments{
\item{seed}{The random seed. Defaults to the current system time in seconds.}

\item{datapath}{Full path to a set of three files: datapath.coord, datapath.diffs and datapath.outer.}

\item{mcmcpath}{Full path to a filename prefix in an output directory with write permission.}

\item{prevpath}{Full path to previous output directory, i.e., the mcmcpath in a previous EEMS run. Optional.}

\item{gridpath}{Full path to a set of two files: gridpath.demes and gridpath.edges. Optional.}

\item{nDemes}{Number of demes, roughly. EEMS constructs a regular triangular grid with circa nDemes vertices.}

\item{nIndiv}{Number of samples. Should match the size of the dissimilarity matrix in datapath.diffs.}

\item{nSites}{Number of SNPs used to compute the observed dissimilarity matrix in datapath.diffs.}

\item{diploid}{Logical value that indicates whether the species is diploid (TRUE) or haploid (FALSE). Defaults to TRUE.}

\item{distance}{Distance metric. Either \code{euclidean} or \code{greatcirc}, that is, great circle distance.
Defaults to 'euclidean'.}

\item{numMCMCIter}{Number of Markov Chain Monte Carlo iterations.}

\item{numBurnIter}{Number of burn-in iterations to be discarded before sampling from posterior.}

\item{numThinIter}{Number of thinning iterations to be discarded between sampling from posterior.}

\item{mSeedsProposalS2}{Variance of normal proposals to update the seeds of the migration tiles. Defaults to 0.01.}

\item{qSeedsProposalS2}{Variance of normal proposals to update the seeds of the diversity tiles. Defaults to 0.1.}

\item{mEffctProposalS2}{Variance of normal proposals to update the log10 rates of the migration tiles. Defaults to 0.1.}

\item{qEffctProposalS2}{Variance of normal proposals to update the log10 rates of the diversity tiles. Defaults to 0.001.}

\item{mrateMuProposalS2}{Variance of normal proposals to update the overall mean migration rate, on the log10 scale.
Defaults to 0.01.}

\item{qVoronoiPr}{With probability qVoronoiPr, update diversity Voronoi;
with probability 1-qVoronoiPr, update migration Voronoi. Defaults to 0.25.}

\item{qrateShape}{Shape hyperparameter for the diversity rates variance, qrateS2 ~ invgamma(qrateShape, qrateScale).
Defaults to 0.001}

\item{mrateShape}{Shape hyperparameter for the migration rates variance, mrateS2 ~ invgamma(mrateShape, mrateScale).
Defaults to 0.001.}

\item{sigmaShape}{Shape hyperparameter for the scaling factor sigma^2 ~ invgamma(sigmaShape, sigmaScale).
Defaults to 0.001.}

\item{qrateScale}{Scale hyperparameter for the diversity rates variance, qrateS2 ~ invgamma(qrateShape, qrateScale).
Defaults to 1.0.}

\item{mrateScale}{Scale hyperparameter for the migration rates variance, mrateS2 ~ invgamma(mrateShape, mrateScale).
Defaults to 1.0.}

\item{sigmaScale}{Scale hyperparameter for the scaling factor sigma^2 ~ invgamma(sigmaShape, sigmaScale).
Defaults to 1.0.}

\item{negBiProb}{Success probability for the number of Voronoi tiles ~ negbinom(negBiSize, negBiProb). Defaults to 0.67.}

\item{negBiSize}{Size for the number of Voronoi tiles ~ negbinom(negBiSize, negBiProb). Defaults to 10.}
}
\value{
None
}
\description{
This function runs an EEMS analysis with given parameters, which include file paths for
retrieving the input files (datapath.coord, datapath.diffs and datapath.outer) and
writing the output files.
It is an exact replicate of the original EEMS command-line function.
}
\examples{
# We use example input from Petkova et al. (2016), found in the '/extdata' directory
data_path <- system.file("extdata", package = "reems")
input <- file.path(data_path, "barrier-schemeX-nIndiv300-nSites3000")
# The example puts the output in a temporary directory.
mcmcdir <- file.path(tempdir(), "eems_out")
dir.create(mcmcdir, showWarnings = FALSE)


# Run an example EEMS analysis with a small number of iterations to ensure quick termination. 

eems.from.files(
  datapath = input,
  mcmcpath = mcmcdir,
  nDemes = 200,
  nIndiv = 300,
  nSites = 3000,
  diploid = FALSE,
  numMCMCIter = 200,
  numBurnIter = 100,
  numThinIter = 99,
)
# Delete the output directory to tidy up. 
unlink(mcmcdir, recursive = TRUE, force = TRUE)
}
\references{
Petkova, D., Novembre, J. & Stephens, M.
Visualizing spatial population structure with estimated effective migration surfaces.
Nat Genet 48, 94–100 (2016). https://doi.org/10.1038/ng.3464
}
\seealso{
\code{\link{eems.plots}, \link{eems}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{outer.buffered}
\alias{outer.buffered}
\title{Create a buffered habitat around a set of points.}
\usage{
outer.buffered(coords, d = NULL)
}
\arguments{
\item{coords}{A two-column matrix of point coordinates (longitude, latitude).}

\item{d}{Rough buffer distance. Not necessarily kept completely to keep the polygon simple.
Defaults to 0.2 of the length of the hull of the \code{coords}.}
}
\value{
A two-column matrix of coordinates with the outer boundary points of the containing polygon.
}
\description{
Creates a polygon a certain distance from each of a set of points.
It is used for creating a default habitat outline for \code{\link[=eems]{eems()}}.
}
\examples{
#  Create a buffer around the example habitat
outer <- outer.buffered(ex.coords)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ex.coords}
\alias{ex.coords}
\title{Example coordinate data from ConStruct}
\format{
\subsection{\code{ex.coords}}{

A matrix with 16 rows and 2 columns.
The rows designate samples and the two columns store the X and Y coordinate respectively. .
}
}
\source{
https://github.com/gbradburd/conStruct/tree/master/data
}
\usage{
ex.coords
}
\description{
Example coordinate data from ConStruct
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eems.run.R
\name{eems}
\alias{eems}
\title{Run an EEMS analysis from data in R format}
\usage{
eems(
  seed = unclass(Sys.time()),
  coords,
  freqs,
  outer = NULL,
  buffer = NULL,
  mcmcpath,
  demes = NULL,
  edges = NULL,
  prevpath = "",
  nChains = 1,
  parallel = FALSE,
  nWorkers = NULL,
  nDemes = NULL,
  diploid = TRUE,
  distance = "greatcircle",
  numMCMCIter = 2e+06,
  numBurnIter = 1e+06,
  numThinIter = 9999,
  mSeedsProposalS2 = 0.01,
  qSeedsProposalS2 = 0.1,
  mEffctProposalS2 = 0.1,
  qEffctProposalS2 = 0.001,
  mrateMuProposalS2 = 0.01,
  qVoronoiPr = 0.25,
  qrateShape = 0.001,
  mrateShape = 0.001,
  sigmaShape = 0.001,
  qrateScale = 1,
  mrateScale = 1,
  sigmaScale = 1,
  negBiProb = 0.67,
  negBiSize = 10
)
}
\arguments{
\item{seed}{The random seed. Defaults to the current system time in seconds.}

\item{coords}{The coordinate matrix, a two-column matrix with a line for every sample.}

\item{freqs}{An allele frequency  matrix in \code{conStruct} format, with a column for every site and a row for every sample.
Choosing one main allele, it tabulates the average genotypes of the sample at that locus.}

\item{outer}{Matrix of coordinates of the outer polygon.
Defaults to a polygon larger than the convex hull of the supplied coordinates to mitigate EEMS boundary effects,
which is created by \code{\link[=outer.buffered]{outer.buffered()}}.}

\item{buffer}{The rough distance of the boundary from the convex hull. Defaults to 0.2 times the length of the hull.
Unused if \code{outer} is specified.}

\item{mcmcpath}{Full path to a filename prefix in an output directory with write permission.}

\item{demes}{Matrix of coordinates of demes; optional, and usually unnecessary. Only in conjunction with \code{edges}.}

\item{edges}{Matrix of custom input grid; optional, and usually unnecessary. Only in conjunction with \code{demes}.}

\item{prevpath}{Full path to previous output directory, i.e., the mcmcpath in a previous EEMS run. Optional.}

\item{nChains}{Number of chains to be run. Defaults to 1.}

\item{parallel}{Logical values that indicates whether chains should be run in parallel. Defaults to FALSE.}

\item{nWorkers}{If \code{parallel} is TRUE, then the number of workers used is the minimum of \code{nChains} and \code{nWorkers}.
Defaults to the number of available cores.}

\item{nDemes}{Number of demes, roughly. EEMS constructs a regular triangular grid with circa nDemes vertices.
Defaults to 6 times the number of individuals, aiming for sufficient resolution.}

\item{diploid}{Logical value that indicates whether the species is diploid (TRUE) or haploid (FALSE). Defaults to TRUE.}

\item{distance}{Distance metric. Either \code{euclidean} or \code{greatcirc}, that is, great circle distance.
Defaults to 'greatcirc', which is more accurate at larger scales.}

\item{numMCMCIter}{Number of Markov Chain Monte Carlo iterations. Defaults to 2000000.}

\item{numBurnIter}{Number of burn-in iterations to be discarded before sampling from posterior. Defaults to 1000000.}

\item{numThinIter}{Number of thinning iterations to be discarded between sampling from posterior. Defaults to 9999.}

\item{mSeedsProposalS2}{Variance of normal proposals to update the seeds of the migration tiles. Defaults to 0.01.}

\item{qSeedsProposalS2}{Variance of normal proposals to update the seeds of the diversity tiles. Defaults to 0.1.}

\item{mEffctProposalS2}{Variance of normal proposals to update the log10 rates of the migration tiles. Defaults to 0.1.}

\item{qEffctProposalS2}{Variance of normal proposals to update the log10 rates of the diversity tiles. Defaults to 0.001.}

\item{mrateMuProposalS2}{Variance of normal proposals to update the overall mean migration rate, on the log10 scale.
Defaults to 0.01.}

\item{qVoronoiPr}{With probability qVoronoiPr, update diversity Voronoi;
with probability 1-qVoronoiPr, update migration Voronoi. Defaults to 0.25.}

\item{qrateShape}{Shape hyperparameter for the diversity rates variance, qrateS2 ~ invgamma(qrateShape, qrateScale).
Defaults to 0.001}

\item{mrateShape}{Shape hyperparameter for the migration rates variance, mrateS2 ~ invgamma(mrateShape, mrateScale).
Defaults to 0.001.}

\item{sigmaShape}{Shape hyperparameter for the scaling factor sigma^2 ~ invgamma(sigmaShape, sigmaScale).
Defaults to 0.001.}

\item{qrateScale}{Scale hyperparameter for the diversity rates variance, qrateS2 ~ invgamma(qrateShape, qrateScale).
Defaults to 1.0.}

\item{mrateScale}{Scale hyperparameter for the migration rates variance, mrateS2 ~ invgamma(mrateShape, mrateScale).
Defaults to 1.0.}

\item{sigmaScale}{Scale hyperparameter for the scaling factor sigma^2 ~ invgamma(sigmaShape, sigmaScale).
Defaults to 1.0.}

\item{negBiProb}{Success probability for the number of Voronoi tiles ~ negbinom(negBiSize, negBiProb). Defaults to 0.67.}

\item{negBiSize}{Size for the number of Voronoi tiles ~ negbinom(negBiSize, negBiProb). Defaults to 10.}
}
\value{
A vector of output directories which can be given as input to \code{\link[=eems.plots]{eems.plots()}}.
}
\description{
This function runs an EEMS analysis with given parameters in the input format for \code{conStruct},
which include R matrices for allele frequency data and coordinates.
In addition to taking R objects as inputs,
it also provides additional default values over the original \code{EEMS} command-line utility.
Furthermore, it allows for running more than one chain, either in parallel or sequentially,
and returns a vector of output directories as input to \code{\link[=eems.plots]{eems.plots()}}.
}
\examples{
# The example puts the output in a temporary directory.
mcmcdir <- file.path(tempdir(), "eems_out")
dir.create(mcmcdir, showWarnings = FALSE)
mcmcpath <- file.path(mcmcdir,"example")

# Run an example EEMS analysis with a small number of iterations to ensure quick termination. 

eems(
  freqs = ex.freqs,
  coords = ex.coords,
  mcmcpath = mcmcpath,   
  numMCMCIter = 200,
  numBurnIter = 100,
  numThinIter = 99,
)
# Delete the output directory to tidy up. 
unlink(mcmcdir, recursive = TRUE, force = TRUE)
}
\references{
Petkova, D., Novembre, J. & Stephens, M.
Visualizing spatial population structure with estimated effective migration surfaces.
Nat Genet 48, 94–100 (2016). https://doi.org/10.1038/ng.3464
}
\seealso{
\code{\link{eems.plots}, \link{eems.from.files}}
}

\name{reems}
\alias{rEEMS}
\docType{package}
\title{
Estimating Effective Migration Surfaces (EEMS)  
}
\description{
    Provides several functions to run EEMS analyses and to visualise the
    results of EEMS. For an exmplanation of and introduction to the EEMS
    method, see Petkova, D., Novembre, J. & Stephens, M. (2016) <doi:10.1038/ng.3464>.

    \code{\link{eems}} is the main entrypoint, running EEMS analyses on
    single nucleotide polymorphism data allele frequeny available as R matrices.

    \code{\link{eems.from.files}} runs an EEMS analysis from input data
    available as tabular files, and correesponds precisely to the
    original EEMS utility available at <https://github.com/dipetkov/eems>.   
    
    \code{\link{eems.plots}} takes the output of one or more EEMS runs
    and produces contour plots of the effective migration and diversity rates,
    as well as scatterplots of observed vs fitted genetic
    dissimilarities and trace plots to assess convergence.
    
    \code{\link{eems.population.grid}} plots the population grid, with all edges in the same color,
    to visualize the grid before estimating migration and diversity rates.
    
    \code{\link{eems.voronoi.samples}} and \code{\link{eems.posterior.draws}} both take random samples
    from the posterior distribution of the migration and diversity rates and plot
    them to visualize the posterior variance (in a slightly different way).
    }
\keyword{ package }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eems.plots.R
\name{eems.plots}
\alias{eems.plots}
\title{A function to plot effective migration and diversity surfaces from EEMS output}
\usage{
eems.plots(
  mcmcpath,
  plotpath,
  longlat,
  plot.width = 10,
  plot.height = 10,
  out.png = NULL,
  res = 600,
  xpd = TRUE,
  add.grid = FALSE,
  col.grid = "gray80",
  lwd.grid = 1,
  add.demes = FALSE,
  col.demes = "black",
  pch.demes = 19,
  min.cex.demes = 1,
  max.cex.demes = 3,
  add.outline = FALSE,
  col.outline = "gray90",
  lwd.outline = 2,
  projection.in = NULL,
  projection.out = NULL,
  add.map = FALSE,
  col.map = "gray60",
  lwd.map = 2,
  eems.colors = NULL,
  prob.levels = c(0.9, 0.95),
  add.colbar = TRUE,
  m.colscale = NULL,
  q.colscale = NULL,
  remove.singletons = TRUE,
  add.abline = FALSE,
  add.r.squared = FALSE,
  add.title = TRUE,
  m.plot.xy = NULL,
  q.plot.xy = NULL,
  xy.coords = NULL
)
}
\arguments{
\item{mcmcpath}{A vector of EEMS output directories, for the same dataset. Warning: There is
minimal checking that the given  directories are for the same dataset.}

\item{plotpath}{The full path and the file name for the graphics to be generated.}

\item{longlat}{A logical value indicating whether the coordinates are given as pairs
(longitude, latitude) or (latitude, longitude).}

\item{plot.width}{The width of the graphics region for the two rate contour plots, in inches.
The default value is 10.}

\item{plot.height}{The height of the graphics region, in inches. The default value is 10.}

\item{out.png}{A logical value which, if set,  forces output graphics to be generated as PNGs
(if \code{TRUE}) or PDFs (if \code{FALSE}). If left unset, the format depends on the nature of the plot.}

\item{res}{Resolution, in dots per inch; used only for PNG images.
The default is 600.}

\item{xpd}{A logical value indicating whether to clip plotting to the figure region
(\code{xpd} = TRUE, which is the default) or clip plotting to the plot region
(\code{xpd} = FALSE).}

\item{add.grid}{A logical value indicating whether to add the population grid or not.}

\item{col.grid}{The color of the population grid. Defaults to \code{gray80}.}

\item{lwd.grid}{The line width of the population grid. Defaults to 1.}

\item{add.demes}{A logical value indicating whether to add the observed demes or not.}

\item{col.demes}{The color of the demes. Defaults to \code{black}.}

\item{pch.demes}{The symbol, specified as an integer, or the character to be used for
plotting the demes. Defaults to 19.}

\item{min.cex.demes}{The minimum size of the deme symbol/character.}

\item{max.cex.demes}{The maximum size of the deme symbol/character. Defaults to 1 and 3,
respectively. If \code{max.cex.demes} > \code{min.cex.demes}, then demes with more samples
also have bigger size: the deme with the fewest samples has size \code{min.cex.demes} and
the deme with the most samples has size \code{max.cex.demes}.}

\item{add.outline}{A logical value indicating whether to add the habitat outline or not.}

\item{col.outline}{The color of the habitat outline. Defaults to \code{white}.}

\item{lwd.outline}{The line width of the habitat outline. Defaults to 2.}

\item{projection.in}{The input cartographic projection, specified as a PROJ.4 string.}

\item{projection.out}{The output cartographic projection, specified as a PROJ.4 string.}

\item{add.map}{A logical value indicating whether to add a high-resolution geographic map.
Requires the \code{rworldmap} and \code{rworldxtra} packages. It also requires that
\code{projection.in} is specified.}

\item{col.map}{The color of the geographic map. Default is \code{gray60}.}

\item{lwd.map}{The line width of the geographic map. Defaults to 2.}

\item{eems.colors}{The EEMS color scheme as a vector of colors, ordered from low to high.
Defaults to a DarkOrange to Blue divergent palette with six orange shades, white in the middle,
six blue shades. Acknowledgement: The default color scheme is adapted from the \code{dichromat}
package.}

\item{prob.levels}{A vector of probabilities for plotting the posterior probability contours of
\code{P(m > 0 | diffs)} and \code{P(m < 0 | diffs)}. Defaults to \code{c(0.9, 0.95)}.}

\item{add.colbar}{A logical value indicating whether to add the color bar (the key that shows
how colors map to rates) to the right of the plot. Defaults to TRUE.}

\item{m.colscale}{A fixed range for log10-transformed migration rates. If the estimated rates
fall outside the specified range, then the color scale is ignored. By default, no range is
specified for either type of rates.}

\item{q.colscale}{A fixed range for log10-transformed diversity rates.}

\item{remove.singletons}{Remove demes with a single observation from the diagnostic scatter
plots. Defaults to TRUE.}

\item{add.abline}{Add the line \code{y = x} to the diagnostic scatter plots of observed vs
fitted genetic dissimilarities.}

\item{add.r.squared}{Add the R squared coefficient to the diagnostic scatter plots of observed
vs fitted genetic dissimilarities.}

\item{add.title}{A logical value indicating whether to add the main title in the contour plots.
Defaults to TRUE.}

\item{m.plot.xy}{Statements which add graphical elements (e.g. points) on top of the migration
sufrace.}

\item{q.plot.xy}{Statements which add graphical elements (e.g. points) on top of the diversity
surface.}

\item{xy.coords}{Additional coordinates at which to estimate the migration and diversity rates.}
}
\value{
None
}
\description{
Given a vector of EEMS output directories, this function generates several figures
to visualize EEMS results. It is a good idea to examine all these figures, which is
why they are generated by default.
\itemize{
\item \code{plotpath-mrates01}: effective migration surface. This contour plot visualizes
the estimated effective migration rates \code{m}, on the log10 scale after mean centering.
\item \code{plotpath-mrates02}: posterior probabilities \code{P(m > 0 | diffs)} and
\code{P(m < 0 | diffs)} for each location in the habitat. Since migration rates are
visualized on the log10 scale after mean centering, 0 corresponds to the overall mean
migration rate. This contour plot emphasizes regions with effective migration that is
significantly higher/lower than the overall average.
\item \code{plotpath-qrates01}: effective diversity surface. This contour plot visualizes
the estimated effective diversity rates \code{q}, on the log10 scale after mean centering.
\item \code{plotpath-qrates02}: posterior probabilities \code{P(q > 0 | diffs)} and
\code{P(q < 0 | diffs)}. Similar to \code{plotpath-mrates02} but applied to the effective
diversity rates.
\item \code{plotpath-rdist01}: scatter plot of the observed vs the fitted between-deme
component of genetic dissimilarity, where one point represents a pair of sampled demes.
\item \code{plotpath-rdist02}: scatter plot of the observed vs the fitted within-deme
component of genetic dissimilarity, where one point represents a sampled deme.
\item \code{plotpath-rdist03}: scatter plot of observed genetic dissimilarities between
demes vs observed geographic distances between demes.
\item \code{plotpath-pilogl01}: posterior probability trace
}
The \code{mrates} and \code{qrates} figures visualize (properties of) the effective migration
and diversity rates across the habitat. The other figures can help to check that the MCMC
sampler has converged (the trace plot \code{pilogl}) and that the EEMS model fits the data
well (the scatter plots of genetic dissimilarities \code{rdist}).
}
\details{
The function \code{eems.plots} will work given the results from a single EEMS run (one
directory in \code{mcmcpath}) but it is better to run EEMS several times, randomly initializing
the MCMC chain each time. In other words, simulate several realizations of the Markov chain
and let each realization start from a different state in the parameter space (by using a
different random seed).

Detail about the within-deme and between-deme components of genetic dissimilarity: Let
\code{D(a,b)} be the dissimilarity between one individual from deme \code{a} and another
individual from deme \code{b}. Then the within-deme component for \code{a} and \code{b}
is simply \code{D(a,a)} and \code{D(b, b)}, respectively. The between-deme component is
\code{D(a,b) - [D(a,a) + D(b,b)] / 2} and it represents dissimilarity that is due to the spatial
structure of the population and is not a consequence of the local diversity in the two demes.
}
\examples{
# Use the provided example or supply the path to your own EEMS run.
extdata_path <- system.file("extdata", package = "reems")
eems_results <- file.path(extdata_path, "EEMS-example")
# Create a temporary output directory for the sake of the example
outdir <- file.path(tempdir(), "plot_out")
dir.create(outdir, showWarnings = FALSE) 
name_figures <- file.path(outdir, "EEMS-example")

# Produce the set of EEMS figures, with default values for all optional parameters.
eems.plots(
  mcmcpath = eems_results,
  plotpath = paste0(name_figures, "-default"),
  longlat = TRUE,
  out.png = FALSE
)

# Delete the temporary output directory to tidy up. 
unlink(outdir, recursive = TRUE, force = TRUE)
}
\references{
Light A and Bartlein PJ (2004). The End of the Rainbow? Color Schemes for Improved
Data Graphics. EOS Transactions of the American Geophysical Union, 85(40), 385.
}
\seealso{
\code{\link{eems.voronoi.samples}, \link{eems.posterior.draws}, \link{eems.population.grid}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eems.voronoi.samples.R
\name{eems.voronoi.samples}
\alias{eems.voronoi.samples}
\title{A function to plot raw Voronoi diagrams of effective migration and diversity rates}
\usage{
eems.voronoi.samples(
  mcmcpath,
  plotpath,
  longlat,
  post.draws = 1,
  plot.width = 10,
  plot.height = 10,
  out.png = FALSE,
  res = 600,
  add.grid = FALSE,
  col.grid = "gray80",
  lwd.grid = 1,
  add.outline = TRUE,
  col.outline = "gray80",
  lwd.outline = 2,
  add.demes = FALSE,
  col.demes = "gray80",
  pch.demes = 19,
  cex.demes = 1,
  add.seeds = TRUE,
  col.seeds = "#8AE234",
  pch.seeds = 4,
  cex.seeds = 1,
  eems.colors = NULL,
  m.colscale = NULL,
  q.colscale = NULL,
  add.title = FALSE
)
}
\arguments{
\item{mcmcpath}{A vector of EEMS output directories, for the same dataset. Warning: There is
minimal checking that the given  directories are for the same dataset.}

\item{plotpath}{The full path and the file name for the graphics to be generated.}

\item{longlat}{A logical value indicating whether the coordinates are given as pairs
(longitude, latitude) or (latitude, longitude).}

\item{post.draws}{Number of times to sample from the posterior. The default is 1.}

\item{plot.width}{The width of the graphics region for the two rate contour plots, in inches.
The default value is 10.}

\item{plot.height}{The height of the graphics region, in inches. The default value is 10.}

\item{out.png}{A logical value which, if set,  forces output graphics to be generated as PNGs
(if \code{TRUE}) or PDFs (if \code{FALSE}). Defaults to \code{FALSE}.}

\item{res}{Resolution, in dots per inch; used only for PNG images.
The default is 600.}

\item{add.grid}{A logical value indicating whether to add the population grid or not.}

\item{col.grid}{The color of the population grid. Defaults to \code{gray80}.}

\item{lwd.grid}{The line width of the population grid. Defaults to 1.}

\item{add.outline}{A logical value indicating whether to add the habitat outline or not.}

\item{col.outline}{The color of the habitat outline. Defaults to \code{white}.}

\item{lwd.outline}{The line width of the habitat outline. Defaults to 2.}

\item{add.demes}{A logical value indicating whether to add the observed demes or not.}

\item{col.demes}{The color of the demes. Defaults to \code{black}.}

\item{pch.demes}{The symbol, specified as an integer, or the character to be used for
plotting the demes. Defaults to 19.}

\item{cex.demes}{The size of the symbol/character used for plotting observed demes.
Defaults to 1.}

\item{add.seeds}{A logical value indicating whether to add the Voronoi seeds or not.}

\item{col.seeds}{The color of the Voronoi seeds. Defaults to \code{green}.}

\item{pch.seeds}{The symbol, specified as an integer, or the character to be used for
plotting the Voronoi seeds. Defaults to 4.}

\item{cex.seeds}{The size of the symbol/character used for plotting the Voronoi seeds.
Defaults to 1.}

\item{eems.colors}{The EEMS color scheme as a vector of colors, ordered from low to high.
Defaults to a DarkOrange to Blue divergent palette with six orange shades, white in the middle,
six blue shades. Acknowledgement: The default color scheme is adapted from the \code{dichromat}
package.}

\item{m.colscale}{A fixed range for log10-transformed migration rates. If the estimated rates
fall outside the specified range, then the color scale is ignored. By default, no range is
specified for either type of rates.}

\item{q.colscale}{A fixed range for log10-transformed diversity rates.}

\item{add.title}{A logical value indicating whether to add the main title in the contour plots.
Defaults to TRUE.}
}
\value{
None
}
\description{
Given a set of EEMS output directories, this function takes random draws from the posterior
distribution of the migration and diversity rate parameters. Each draw is visualized as two
raw Voronoi diagrams; the migration diagram is saved to a file ending in \code{mvoronoiXX},
the diversity diagram is saved to a file ending in \code{qvoronoiXX} where \code{XX} is
a numeric id. Specify the number of times to draw from the posterior with the argument
\code{post.draws}. If \code{post.draws = 10}, then \code{eems.voronoi.samples} will generate
plots with id \code{XX = 1} to \code{XX = 10}.
This function differs from \code{\link[=eems.posterior.draws]{eems.posterior.draws()}} by displaying raw, unsmoothed Voronoi diagrams.
}
\details{
Note about the implementation: \code{eems.voronoi.samples} samples randomly from the posterior
draws saved during the execution of EEMS, after burn-in and thinning.
}
\examples{
# Use the provided example or supply the path to your own EEMS run.
extdata_path <- system.file("extdata", package = "reems")
eems_results <- file.path(extdata_path, "EEMS-example")
# Create a temporary output directory for this example 
outdir <- file.path(tempdir(), "path_out")
dir.create(outdir, showWarnings = FALSE)
name_figures <- file.path(outdir, "eemsplot_out")

# Plot a series of Voronoi diagrams for the EEMS model parameters:
# the effective migration rates (m) and the effective diversity rates (q).
eems.voronoi.samples(
  mcmcpath = eems_results,
  plotpath = paste0(name_figures, "-voronoi-diagrams"),
  longlat = TRUE,
  post.draws = 2,
  out.png = FALSE
)
# Delete the output directory to tidy up. 
unlink(outdir, recursive = TRUE, force = TRUE)
}
\references{
Light A and Bartlein PJ (2004). The End of the Rainbow? Color Schemes for
Improved Data Graphics. EOS Transactions of the American Geophysical Union, 85(40), 385.
}
\seealso{
\code{\link{eems.posterior.draws}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eems.posterior.draws.R
\name{eems.posterior.draws}
\alias{eems.posterior.draws}
\title{Plotting smoothed Voronoi diagrams of effective migration and diversity rates}
\usage{
eems.posterior.draws(
  mcmcpath,
  plotpath,
  longlat,
  post.draws = 1,
  plot.width = 10,
  plot.height = 10,
  out.png = TRUE,
  res = 600,
  xpd = TRUE,
  add.grid = FALSE,
  col.grid = "gray80",
  lwd.grid = 1,
  add.demes = FALSE,
  col.demes = "black",
  pch.demes = 19,
  min.cex.demes = 1,
  max.cex.demes = 3,
  add.outline = FALSE,
  col.outline = "gray90",
  lwd.outline = 2,
  projection.in = NULL,
  projection.out = NULL,
  add.map = FALSE,
  col.map = "gray60",
  lwd.map = 2,
  eems.colors = NULL,
  add.colbar = FALSE,
  m.colscale = NULL,
  q.colscale = NULL,
  add.title = FALSE,
  m.plot.xy = NULL,
  q.plot.xy = NULL
)
}
\arguments{
\item{mcmcpath}{A vector of EEMS output directories, for the same dataset. Warning: There is
minimal checking that the given  directories are for the same dataset.}

\item{plotpath}{The full path and the file name for the graphics to be generated.}

\item{longlat}{A logical value indicating whether the coordinates are given as pairs
(longitude, latitude) or (latitude, longitude).}

\item{post.draws}{Number of times to sample from the posterior. The default is 1.}

\item{plot.width}{The width of the graphics region for the two rate contour plots, in inches.
The default value is 10.}

\item{plot.height}{The height of the graphics region, in inches. The default value is 10.}

\item{out.png}{A logical value which, if set,  forces output graphics to be generated as PNGs
(if \code{TRUE}) or PDFs (if \code{FALSE}). Defaults to \code{TRUE}.}

\item{res}{Resolution, in dots per inch; used only for PNG images.
The default is 600.}

\item{xpd}{A logical value indicating whether to clip plotting to the figure region
(\code{xpd} = TRUE, which is the default) or clip plotting to the plot region
(\code{xpd} = FALSE).}

\item{add.grid}{A logical value indicating whether to add the population grid or not.}

\item{col.grid}{The color of the population grid. Defaults to \code{gray80}.}

\item{lwd.grid}{The line width of the population grid. Defaults to 1.}

\item{add.demes}{A logical value indicating whether to add the observed demes or not.}

\item{col.demes}{The color of the demes. Defaults to \code{black}.}

\item{pch.demes}{The symbol, specified as an integer, or the character to be used for
plotting the demes. Defaults to 19.}

\item{min.cex.demes}{The minimum size of the deme symbol/character.}

\item{max.cex.demes}{The maximum size of the deme symbol/character. Defaults to 1 and 3,
respectively. If \code{max.cex.demes} > \code{min.cex.demes}, then demes with more samples
also have bigger size: the deme with the fewest samples has size \code{min.cex.demes} and
the deme with the most samples has size \code{max.cex.demes}.}

\item{add.outline}{A logical value indicating whether to add the habitat outline or not.}

\item{col.outline}{The color of the habitat outline. Defaults to \code{white}.}

\item{lwd.outline}{The line width of the habitat outline. Defaults to 2.}

\item{projection.in}{The input cartographic projection, specified as a PROJ.4 string.}

\item{projection.out}{The output cartographic projection, specified as a PROJ.4 string.}

\item{add.map}{A logical value indicating whether to add a high-resolution geographic map.
Requires the \code{rworldmap} and \code{rworldxtra} packages. It also requires that
\code{projection.in} is specified.}

\item{col.map}{The color of the geographic map. Default is \code{gray60}.}

\item{lwd.map}{The line width of the geographic map. Defaults to 2.}

\item{eems.colors}{The EEMS color scheme as a vector of colors, ordered from low to high.
Defaults to a DarkOrange to Blue divergent palette with six orange shades, white in the middle,
six blue shades. Acknowledgement: The default color scheme is adapted from the \code{dichromat}
package.}

\item{add.colbar}{A logical value indicating whether to add the color bar (the key that shows
how colors map to rates) to the right of the plot. Defaults to TRUE.}

\item{m.colscale}{A fixed range for log10-transformed migration rates. If the estimated rates
fall outside the specified range, then the color scale is ignored. By default, no range is
specified for either type of rates.}

\item{q.colscale}{A fixed range for log10-transformed diversity rates.}

\item{add.title}{A logical value indicating whether to add the main title in the contour plots.
Defaults to TRUE.}

\item{m.plot.xy}{Statements which add graphical elements (e.g. points) on top of the migration
sufrace.}

\item{q.plot.xy}{Statements which add graphical elements (e.g. points) on top of the diversity
surface.}
}
\value{
None
}
\description{
Given a set of EEMS output directories, this function takes random draws from the posterior
distribution of the migration and diversity rate parameters. Each draw is visualized as two
smoothed Voronoi diagrams; the migration diagram is saved to a file ending in \code{mvoronoiXX},
the diversity diagram is saved to a file ending in \code{qvoronoiXX} where \code{XX} is a
numeric id. Specify the number of times to draw from the posterior with the argument
\code{post.draws}. If \code{post.draws = 10}, then \code{eems.posterior.draws} will generate
plots with id \code{XX = 1} to \code{XX = 10}.
This function differs from \code{\link[=eems.voronoi.samples]{eems.voronoi.samples()}} by plotting smoothed contour diagrams
rather than raw Voronoi plots.
}
\details{
Note about the implementation: \code{eems.voronoi.samples} samples randomly from the posterior
draws saved during the execution of EEMS, after burn-in and thinning.
}
\examples{
# Use the provided example or supply the path to your own EEMS run.
extdata_path <- system.file("extdata", package = "reems")
eems_results <- file.path(extdata_path, "EEMS-example")
# Create a temporary output directory for this example 
outdir <- file.path(tempdir(), "path_out")
dir.create(outdir, showWarnings = FALSE)
name_figures <- file.path(outdir, "EEMS-example")

# Plot a series of Voronoi diagrams for the EEMS model parameters:
# the effective migration rates (m) and the effective diversity rates (q).
eems.posterior.draws(
  mcmcpath = eems_results,
  plotpath = paste0(name_figures, "-posterior-draws"),
  longlat = TRUE,
  post.draws = 2,
  out.png = FALSE
)
# Delete the output directory to tidy up. 
unlink(outdir, recursive = TRUE, force = TRUE)
}
\references{
Light A and Bartlein PJ (2004). The End of the Rainbow? Color Schemes for Improved
Data Graphics. EOS Transactions of the American Geophysical Union, 85(40), 385.
}
\seealso{
\code{\link{eems.voronoi.samples}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contour.with.holes.R
\name{grid.make}
\alias{grid.make}
\title{Create a habitat with holes}
\usage{
grid.make(outer, holes = NULL, plotpath = NULL, nDemes)
}
\arguments{
\item{outer}{A two-column matrix of coordinates of boundary points (longitude, latitude).}

\item{holes}{List of holes. Each hole is a two-column matrix of coordinates of boundary points (longitude, latitude).
This is optional, and defaults to no holes.}

\item{plotpath}{Output filename for the visualisation PNG file. Optional.}

\item{nDemes}{Number of demes, roughly, in the grid without accounting for possible holes.}
}
\value{
A two-entry list of \code{demes} and \code{edges}, both two-column matrices.
}
\description{
\code{grid.make()} constructs a regular triangular grid with circa nDemes vertices. This function creates three files:
\itemize{
\item The pair gridpath.demes and gridpath.edges which specify a custom population grid with holes
\item The plot gridpath.png which visualizes the custom grid with demes in red and edges in green
}
}
\examples{
# We use example input from Petkova et al. (2016), found in the '/extdata' directory
data_path <- system.file("extdata", package = "reems")
input <- file.path(data_path, "barrier-schemeX-nIndiv300-nSites3000")
# Load the population habitat
outer <- read.table(paste0(input, ".outer"))
# Each hole is a ring (simple closed polygon) and the holes don't overlap
hole1 <- data.frame(V1 = c(2., 5., 5., 2., 2.), V2 = c(2., 2., 5., 5., 2.))
hole2 <- data.frame(V1 = c(6.5, 10., 8., 6.5), V2 = c(2.5, 5., 5., 2.5))

# Create the new grid  with holes
grid.make(outer = outer, holes = list(hole1, hole2), nDemes = 300)
}
